/*
 * ata-stuff.h - Internal ATA definitions
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 08/09/01
 *
 * Last CVS commit : $Id: ata-stuff.h,v 1.6 2001/09/12 12:42:10 zion Exp $
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307,
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 * 
 */

// Ports
#define DATA_REG	0			// 16 (or 32) bits wata!! :)
#define ERROR_REG	1
#define FEAT_REG	1
#define SECT_COUNT	2
#define SECT_NUM	3
#define CYL_LSB		4
#define CYL_MSB		5
#define DRV_HEAD	6
#define STATUS		7
#define CMD_REG		7
#define ALT_STATUS	0x206	// same as STATUS, but doesn't reset the IRQ
#define DOR		0x206

// Commands
#define ATA_CMD_RECALIB		0x10
#define ATA_CMD_READ		0x20
#define ATA_CMD_WRITE		0x30
#define ATA_CMD_SEEK		0x70
#define ATA_CMD_DIAG		0x90
#define ATA_CMD_SET_DRV_PARAM	0x91
#define ATA_CMD_MULTI_READ	0xC4
#define ATA_CMD_MULTI_WRITE	0xC5
#define ATA_CMD_SET_MULTI	0xC6
#define ATA_CMD_LOCK_DRV_DOOR	0xDE
#define ATA_CMD_UNLOCK_DRV_DOOR	0xDF
#define ATA_CMD_ID_DRV		0xEC
#define ATA_CMD_SET_FEATURE	0xEF

/* struct: HDDReq
 * desc: The internal structure for a HDD command request.
 */
struct HDDReq {
	__uint8 drive;
	__uint8 command;
	__uint8 len;
	ulong start;
	void (*handler)();
	char *buf;
	struct HDDReq *next;
};

/* struct: drvInfo
 * desc: This struct describes the 512 bytes returned by the ATA_CMD_DIAG macro
 * and all the different data that is contained in it. Struct drvStr is about
 * the only time you'd need to use this, as the rest should be parsed internally
 * into nice functions etc..
 */
struct drvInfo {
/*00*/	__uint16	res	:1,
			hardSect:1,		// Hard sector'd
			softSect:1,		// Soft sector'd
			crap0	:1,
			HSD	:1,		// Head switch delay = 15us (??)
			PDMode	:1,		// Power-down mode implimented
			HDD	:1,		// Hard disk drive
			RDS	:1,		// Removable storage device
			TransRate:3,		// Internal transfer rate
			notebook:1,		// Notebook (rotation deviation)
			crap1	:4,
/*01*/			cyls,			// Physical cylinders
/*02*/			res2,
/*03*/			heads,			// Heads
/*04*/			unformattedBytesPerTrack,
/*05*/			unformattedBytesPerSect,
/*06*/			sects,
/*07*/			vendor0,
/*08*/			vendor1,
/*09*/			vendor2;
/*10*/	__uint8		serial[20];		// Serial num
/*20*/	__uint16	bufferType,		// buffer type (1 = 1-way,
						//	2 = 2-way, 3 = cache)
/*21*/			bufferSize,		// buffer size /512
/*22*/			ECCBytes;		// Number of ECC bytes per-sect
/*23*/	__uint8		firmware[8],		// Firmware revision
/*27*/			model[40],		// Model stuff
/*47*/			sectsPerIntr,		// Sectors read per-interrupt
						//	(multi-sector read mode)
			vendor3;
/*48*/	__uint16	longIO;			// 32bit IO enabled?
/*49*/	__uint8		vendor4,
			DMA	:1,		// DMA transfers allowed?
			LBA	:1,		// LBA mode allowed?
			crap3	:6;
/*50*/	__uint16	res50;
/*51*/	__uint8		vendor5,
			PIOCycTime,		// PIO cycle time
/*52*/			vendor6,
			DMACycTime;		// DMA cycle time
/*53*/	__uint16	fieldValid,
/*54*/			logCyls,		// Logical cylinders
/*55*/			logHeads,		// Logical heads
/*56*/			logSects;		// Logical sectors
/*57*/	__uint32	logBytesPerSect;	// bytes-per-logical-sector
/*59*/	__uint8		sectsPerIntrN,		// Number of sectors between
						//	two intrrupts
			mulSectOK;		// Multiple sector reads are OK?
/*60*/	__uint32	LBACap;			// LBA capacity (in sects)
/*62*/	__uint8		DMASingSup,		// Single DMA modes supported
			DMASingAct,		// Single DMA mode active
/*63*/			DMAMultSup,		// Multiple DMA modes supported
			DMAMultAct;		// Multiple DMA mode active
/*64*/	__uint16	EATAPIOModes,		// PIO modes in EATA
/*65*/			EATADMAMin,		// min DMA cycle-time (ns)
/*66*/			EATADMATime,		// recomended DMA cycle-time(ns)
/*67*/			EATAPIO,		// PIO cycle time (w/o IORDY)
/*68*/			EATAPIOIORDY,		// PIO cycle time (w/ IORDY)
/*69*/			res69[6],
/*75*/			queueDepth,		// ?
/*76*/			res76[4],
/*80*/			ATAMajor,		// Major revision number
/*81*/			ATAMinor,		// Minor revision number
/*82*/			cmdSet[2],		// Extra command sets?
/*84*/			cmdSetFeatExten,	// CMD_SET_FEAT extensions
						//	supported?
/*85*/			csfEnable[2],		// CMD_SET_FEAT options
/*87*/			cfsDefault,		// CMD_SET_FEAT defaults
/*88*/			UDMA,			// UDMA modes supported
/*89*/			res89,
/*90*/			res90,
/*91*/			APMVals,		// Values used for APM
/*92*/			res92,
/*93*/			hwConfig;		// How the hw is config'd
/*94*/	__uint16	end[162];		// Pad the rest up to 1 block
} __PACKED__;

/* struct: drvStr
 * desc: Info on a per-drive basis
 */
struct drvStr {
	__uint32 timeout;		// what the timeout should be
	struct drvInfo info;		// Info!
	__uint32 capacity;		// Capacity in sectors
	bool init;			// Has the drive been init'd yet? 0
					// for no, 1 for yes
	__uint8 ATALev;			// Level of ATA compliance
	__uint8 sectsPerIntr;		// Number of sectors to rw per-interrupt
};

/* struct: cntStr
 * desc: Info on a per-controler basis
 */
struct cntStr {
	volatile bool done;		// done?
	volatile bool doing;		// are we doing anything?
	char IRQ;			// IRQ
	char req;			// Request currently going on
	__uint8 curDrv;			// Current drive config'd
	__uint8 num;
	__uint8 cache[5];		// Cache of the registers of this
					// controller (should speed up output)
	bool cacheValid;		// Is the cache valid?
	uint addr;			// The base addy of the controller
	void *dataBlock;		// Where the data is transfered to/from
	void (*handler)();		// Current interrupt handler
	struct drvStr *drive[ATA_MAX_DRV];	// All the drives
	__uint32 timeout;		// Timeout of the controller (FIXME?)
	ulong sectsToDo;		// Number of sectors left to process
};

// Drive functions:
// Cute little macro to make things easier
static inline __uint8 drvBT(struct cntStr *cnt, ulong reg, __uint8 bit)
{
	return inb(cnt->addr + reg) & bit;
}

#define isPollBusy(cnt)		drvBT(cnt, ALT_STATUS, 0x80)
#define isPollReady(cnt)	drvBT(cnt, ALT_STATUS, 0x40)
#define isPollWriteFault(cnt)	drvBT(cnt, ALT_STATUS, 0x20)
#define isPollSeeking(cnt)	drvBT(cnt, ALT_STATUS, 0x10)
#define isPollDRQ(cnt)		drvBT(cnt, ALT_STATUS, 0x08)
#define isPollECCCorrected(cnt)	drvBT(cnt, ALT_STATUS, 0x04)
#define isPollError(cnt)	drvBT(cnt, ALT_STATUS, 0x01)
#define getPollStatus(cnt)	drvBT(cnt, ALT_STATUS, 0xFF)

#define isBusy(cnt)		drvBT(cnt, STATUS, 0x80)
#define isReady(cnt)		drvBT(cnt, STATUS, 0x40)
#define isWriteFault(cnt)	drvBT(cnt, STATUS, 0x20)
#define isSeeking(cnt)		drvBT(cnt, STATUS, 0x10)
#define isDRQ(cnt)		drvBT(cnt, STATUS, 0x08)
#define isECCCorrected(cnt)	drvBT(cnt, STATUS, 0x04)
#define isError(cnt)		drvBT(cnt, STATUS, 0x01)
#define getStatus(cnt)		drvBT(cnt, STATUS, 0xFF)

#define aborted(cnt)		drvBT(cnt, ERROR_REG, 0x08)

errRet HDDWait(struct cntStr *, __uint8);
errRet HDDWaitFinish();
void HDDDoDataTransfer();
void ATAIntrHandler();
errRet HDDDoCmd(__uint32, __uint16,void *, deviceCmd, struct cntStr *, __uint8);
errRet HDDReset(struct cntStr *, __uint8);
errRet HDDDetectParam(struct cntStr *, __uint8);
errRet ATAAddDrv(struct cntStr *, __uint8);
void ATADelDrv(struct cntStr *, __uint8);
errRet cntReset(struct cntStr *);
void ATADelCnt(__uint8);
errRet ATAConfCnt(__uint8);
