/*
 * floppy.c - A floppy driver
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Orphan
 * Last update: 30/07/00
 *
 * Last CVS commit : $Id: floppy.c,v 1.5 2001/09/12 12:42:10 zion Exp $
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 * This file should impliment a floppy disk driver but doesn't and is out of
 * date.
 *
 * Notes/Fixme's:
 * The file..
 *
 * Changelog:
 *
 */

#if 0
#include <common.h>
#include <floppy.h>
#include <dma.h>
#include <irq.h>
#include <printk.h>
#include <io.h>
#include <stdio.h>
#include <assert.h>
#include <stdarg.h>
#include <pit.h>
#include <delay.h>

#define NUM_DRIVES 1		// change if you have more :)

// Ports
#define FDC_BASE_0 0x3F0	// Base address for the floppy
#define FDC_BASE_1 0x370	// Base address for the floppy
#define FDC_REG 1		// Offset for the status register from base
#define FDC_DOR 2		// Offset for the Digital Output Register
#define FDC_MSR 4		// the offset of the Main Status Register
#define FDC_DRS 4		// the offset of the Data Rate Select reg
#define FDC_DR  5		// Offset of the Data Register
#define FDC_DIR 7		// Offset of the Digital Input Register
// IRQ
#define FDC_IRQ 6		// IRQ channel
// DMA
#define FDC_DMA 2		// DMA channel

#define FPORT(x) (((curFloppy > 3)?FDC_BASE_0:FDC_BASE_1) + x)

// Other stuff
#define ON 1
#define OFF 0

// Commands
#define CMD_READ_TRACK		0x02
#define CMD_FIX_DRIVE_DATA	0x03
#define CMD_CHK_DRIVE_STATUS	0x04
#define CMD_WRITE_SECT		0x05
#define CMD_READ_SECT		0x06
#define CMD_CALIB_DRIVE		0x07
#define CMD_CHK_INTR_STATUS	0x08
#define CMD_WRITE_DEL_SECT	0x09
#define CMD_READ_SECT_ID	0x0A
#define CMD_READ_DEL_SECT	0x0C
#define CMD_FORMAT_TRACK	0x0D
#define CMD_SEEK		0x0F

// Extended commands
#define CMD_CTRL_VER		0x10

// Bits in the status byte
#define DISK_CHANGE_BIT 0	// 1 is for a media changs
#define DONE_BIT 1		// 1 if it's done

#if NUM_DRIVES > 8
#error Too many disk drives to handle :/
#endif

struct structGeom {
	__uint8 tracks;		// Tracks
	__uint8 heads;		// Heads
	__uint8 sectors;	// Sectors per track
	__uint8 headLoadTime;	// Time taken to load a head
	__uint8 headUnloadTime;	// Time taken to unload a head
	__uint8 stepRate;	// Step Rate
	__uint8 transferRate;	// Rate of transfer
	__uint8 gap3Size;	// the size of gap3
	__uint8 sectorSize;	// Size of the sector
} defaults[] = {
/* If you're interested, please read this comment before looking/changing the
 * following values:
 * transfer rates
 *       |   data transfer rate      |
 *       |  1M  | 500K | 300K | 250K |
 * value |   3  |   0  |   1  |   2  |
 * Sector Size:
 * value | sector size |
 * ------+-------------+
 * 0     |        128b |
 * 1     |        256b |
 * 2     |        512b |
 * ...
 * 7     |        16Kb |
 * Head loading time (ms):
 * value   |   data transfer rate      |
 *         |  1M  | 500K | 300K | 250K |
 * --------+------+------+------+------+
 * 0       | 128  | 256  | 426  | 512  |
 * 1       |   1  |   2  |   3  |   4  |
 * 2       |   2  |   4  |   7  |   8  |
 * 3       |   3  |   6  |  10  |  12  |
 * ..
 * 127     | 127  | 254  | 423  | 508  |
 * 
 * Head unloading time (ms):
 * value   |   data transfer rate      |
 *         |  1M  | 500K | 300K | 250K |
 * --------+------+------+------+------+
 * 0       | 128  | 256  | 426  | 512  |
 * 1       |   8  |  16  |  27  |  32  |
 * 2       |  16  |  32  |  54  |  64  |
 * 3       |  24  |  48  |  81  |  96  |
 * ..
 * 15      | 120  | 240  | 400  | 480  |
 *
 * Step Rate (ms):
 * value   |   data transfer rate      |
 *         |  1M  | 500K | 300K | 250K |
 * --------+------+------+------+------+
 * 0       |   8  |  16  | 26.7 |  32  |
 * 1       | 7.5  |  15  |   25 |  30  |
 * 2       |   7  |  14  | 23.3 |  28  |
 * 3       | 6.5  |  13  | 21.7 |  26  |
 * ..
 * 15      |  .5  |   1  | 400  |   2  |
 * (These tables taken from 'The Indespensable PC hardware book' - Mark)
 * I'm not sure what values to put here. If ya know, please correct them :)
 */
	{80, 1,  8, 0, 0, 0, 2, 27, 2},	//  320Kb 3.5	0
	{80, 2,  8, 0, 0, 0, 2, 27, 2},	//  640Kb 3.5	1
	{80, 2,  9, 0, 0, 0, 2, 27, 2},	//  720Kb 3.5	2
	{80, 2, 18, 0, 0, 0, 0, 27, 2},	// 1.44Mb 3.5	3
	{80, 2, 21, 0, 0, 0, 0, 27, 2},	// 1.65Mb 3.5	4
};

__uint8 DOR;			// Digital Output Register
__uint8 curFloppy;		// the number of the current floppy
__uint8 extended;		// is it an extended controller?
volatile ulong mWait=0;		// how long do we have to wait?
volatile ulong timeout=0;	// timeout?

struct floppy {
	struct structGeom geom;
	char status;		// the status byte
	__uint8 result[7];	// the result bytes
	__uint8 senseInt;	// the byte from senseInterrupt
	__uint8 curTrack;	// The track we're currently on
} floppys[NUM_DRIVES];

// Functions
char getByte();
void putByte(char);
__uint8 issue(char, ...);
void motorSet(__uint8);

/* Waits for the floppy to finish and gets the bytes returned by it (bytes is
 * how many bytes we expect the command to return */
__uint8 floppyWait(char senseInt, char bytes)
{
	char i;
	while(!bt(&floppys[curFloppy].status, DONE_BIT) && timeout)
		/* Wait... */;
	if(!timeout)			// If it times out
		bytes = 7;

	// Else get the returned values
	for(i=0;i<bytes;i++)
		floppys[curFloppy].result[i] = getByte();

	if(senseInt) {
		issue(CMD_CHK_INTR_STATUS);
		floppys[curFloppy].senseInt = getByte();
		floppys[curFloppy].curTrack = getByte();
	}

	if(!timeout) {
		if(inb(FDC_DIR) & 0x80)		// Disk change?
			bts(&floppys[curFloppy].status, DISK_CHANGE_BIT);
		return FALSE;
	}
	return TRUE;
}

/* sorta temp func that is called from below */
__uint8 handleRWReq(__uint8 head, __uint8 track, __uint8 sector, __uint8 length)
{
	putByte((head << 2) + curFloppy);	// Some info
	putByte(track);				// Track
	putByte(head);				// Head
	putByte(sector);			// Sector
	putByte(floppys[curFloppy].geom.sectorSize);
	putByte(length);			// Length
	putByte(floppys[curFloppy].geom.gap3Size);
	putByte(0xFF);				// Not sure what this does...
	return TRUE;
}

/* Issues a command, read track for example, to the floppy controler */
__uint8 issue(char command, ...)
{
	char *args;
	__uint8 t = 0;				// HACK!! :)

	switch(command) {
	// No arguments
	case CMD_FIX_DRIVE_DATA:		// Fix the drive data
		putByte(command);
		putByte((floppys[curFloppy].geom.stepRate << 4) +
				floppys[curFloppy].geom.headUnloadTime);
		putByte((floppys[curFloppy].geom.headLoadTime << 1) + 1);
		return TRUE;

	// __uint8 head, __uint8 track, __uint8 sector, __uint8 length
	case CMD_READ_SECT:
		t = 32;
	case CMD_READ_DEL_SECT:
	case CMD_WRITE_SECT:
	case CMD_WRITE_DEL_SECT:
		vaStartVal(args, 4);
		putByte(command + 64 + t);   // Use MFM (and skip dele'd stuff?)
		return handleRWReq(vaArg(args, __uint8), vaArg(args, __uint8),
				vaArg(args, __uint8), vaArg(args, __uint8));

	// __uint8 head, __uint8 track
	case CMD_READ_TRACK:
		vaStartVal(args, 2);
		putByte(command + 64 + 32);	// Use MFM and skip dele'd stuff
		return handleRWReq(vaArg(args, __uint8), vaArg(args, __uint8),
				0,floppys[curFloppy].geom.sectors);

	// __uint8 head, __uint8 track
	case CMD_SEEK:
		vaStartVal(args, 2);
		putByte(command);
		putByte((vaArg(args, __uint8) << 2) + curFloppy);
		putByte(vaArg(args, __uint8));
		return TRUE;

	// __uint8 head
	case CMD_CHK_DRIVE_STATUS:
		vaStartVal(args, 1);
		putByte(command);
		putByte((vaArg(args, __uint8) << 2) + curFloppy);
		return TRUE;

	// No arguments
	case CMD_CALIB_DRIVE:
		putByte(command);
		putByte(curFloppy);
		return TRUE;
		
	// No arguments
	case CMD_CHK_INTR_STATUS:
		putByte(command);
		return TRUE;

	// __uint8 head, __uint8 track
	case CMD_FORMAT_TRACK:
		vaStartVal(args, 2);
		putByte(command + 64);		// Use MFM
		putByte((vaArg(args, __uint8) << 2) + curFloppy);
		putByte(floppys[curFloppy].geom.sectorSize);
		putByte(floppys[curFloppy].geom.sectors);
		putByte(floppys[curFloppy].geom.gap3Size);
		putByte(0xF6);			// Standard fill byte
		return TRUE;

	// No arguments
	case CMD_CTRL_VER:
		putByte(command);
		return TRUE;

	default:
		return FALSE;
	}
}

__uint8 seek(char track)
{
	__uint8 i;
	assert(track < floppys[curFloppy].geom.tracks);
	if(floppys[curFloppy].curTrack == track)
		return TRUE;

	for(i=0;i<floppys[curFloppy].geom.heads;i++)
		issue(CMD_SEEK, i, track);

	if(!floppyWait(TRUE, 1))	// Timeout :/
		return FALSE;

	sleep(15);			// wait a sec (well, .015 of a sec)
	if((floppys[curFloppy].senseInt != 0x20)
			&& (floppys[curFloppy].curTrack == track))
		return TRUE;
	return FALSE;
}

void floppyRecalib()
{
	motorSet(ON);
	issue(CMD_CALIB_DRIVE);
	floppyWait(TRUE, 0);
	motorSet(OFF);
}

/* turns motor on or off and selects 'curFloppy' */
void motorSet(__uint8 on)
{
	mWait = 0;		// Halt the motor timeout
	assert(curFloppy < NUM_DRIVES);

	// If we're allready on or off
	if((bt(&DOR, curFloppy + 4) && on) || !(bt(&DOR, curFloppy + 4) || on))
		return;

	// Clever: we filtered out the 2 above, so we *can* xor it.
	DOR ^= 1 << (curFloppy + 4);

	// replace the first 2 bits with the new motor we're selecting
	DOR = (DOR & 0xFB) | curFloppy;

	outb(DOR, FPORT(FDC_DOR));

	if(on) {		// If it's being turned on
		sleep(500);	// wait half a sec to spin up
		bts(&floppys[curFloppy].status, DONE_BIT);
	} else
		btc(&floppys[curFloppy].status, DONE_BIT);
}

/* Puts a byte to the floppy controller */
void putByte(char byte)
{
	__uint16 i;
	// Wait till it's ready
	for(i=10000; (i > 0) && !(inb(FPORT(FDC_MSR)) & 0x80); i--)
		/* Wait */;

	if(i) {		// If we didn't fail
		outb(byte, FPORT(FDC_DR));
		return;			// Missed this out, tnx aris :)
	}

	printk(KERROR "FLOPPY WRITE ERROR!!\n");
}

/* Gets a byte to the floppy controller */
char getByte()
{
	__uint16 i;
	// Wait till it's ready
	for(i=10000; (i > 0) && !(inb(FPORT(FDC_MSR)) & 0x80); i--)
		/* Wait */;

	if(i) {		// If we didn't fail
		return inb(FPORT(FDC_DR));
	}

	printk(KERROR "FLOPPY READ ERROR!!\n");
	return FALSE;
}

signalHnd(floppyFinished)
{
	bts(&floppys[curFloppy].status, DONE_BIT);	// We're done!! :)
}

handleInt(floppyTimer)
{
	if(timeout)
		timeout--;

	if(mWait > 1)
		mWait--;
	else if(mWait == 1)
		motorSet(OFF);	// Turn motor off
}

/* resets the floppy disk controller */
void floppyReset()
{
	DOR = 0;			// Disable interrupts and reset
	outb(DOR, FPORT(FDC_DOR));

	motorSet(OFF);

	// Set the transfer rate
	outb(floppys[curFloppy].geom.transferRate, FPORT(FDC_DRS));

	DOR = 0xB;			// Re-enable them
	outb(DOR, FPORT(FDC_DOR));
	// When we reset, we trigger an interrupt...
	if(!floppyWait(TRUE, 0))
		printk(KINFO "floppy.c: RESET floppyWait failed!! :/\n");

	issue(CMD_FIX_DRIVE_DATA);
	seek(1);
	floppyRecalib();
	btc(&floppys[curFloppy].status, DISK_CHANGE_BIT);
}

/* Reads the specified sector */
__uint8 readSect(memory buf, char drive, char sTrack, char sHead,
		char sSect)
{
	char retrys=3;		// Retrys
	for(;retrys;retrys--) {
	}
	return TRUE;
}

__uint8 readSects(memory buf, char drive, char head, char track,
		char sSect, char fSect)
{
	return TRUE;
}

__uint8 readTrack(memory buf, char drive, char head, char track)
{
	return TRUE;
}

/* reads len sectors from floppy drive to memory location buf, starting at
 * track/head/sect */
__uint8 floppyRead(memory buf, char drive, char track, char head,
		char sect, ulong len)
{
	char *b = (char *)buf;
	curFloppy = drive;
	addHandler(6, floppyFinished, 1, NULL);
	addHandler(0, floppyTimer, Hz/100, NULL);	// 100 a sec
	floppyReset();				// Reset it :)

	while(len > floppys[curFloppy].geom.sectors - sect) {
		seek(track);
	}
	delHandler(0, floppyTimer);
	delHandler(6, floppyFinished);
	return TRUE;
}

__uint8 floppyInit()
{
	char i;
	printk(KINFO "Initing the floppy controler: ");
	issue(CMD_CTRL_VER);
	if(getByte() == 0x90) {
		printf("Detected an extended controler\n");
		extended = TRUE;
	} else {
		printf("Detected a normal controler\n");
		extended = FALSE;
	}

	for(i=0;i<NUM_DRIVES;i++) {
		floppys[i].status = 0;
		// Set the done bit
		floppys[i].status |= 1 << DONE_BIT;
		floppys[i].geom = defaults[3];
	}
	curFloppy = 0;
	floppyReset();

	return TRUE;
}
#endif
