/*
 * keyboard.c - A keyboard driver
 *
 * Added to the Zealos project and (C) by Daniel Abraham
 * Also (C) by Reggy
 *
 * Current maintainer: Reggy <reggy@ekky.org>
 * Status: Maintained
 * Last update: 18/10/01
 *
 * Last CVS commit: $Id: keyboard.h,v 1.6 2001/10/18 18:14:00 jalh Exp $
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 * Programed to the keyboard spec found at
 * www.nondot.org/sabre/os/H4HumanInterface/keyboard2.txt.
 *
 * Notes/Fixme's:
 *
 * Changelog:
 * See keyboard.c
 */

#ifndef __generic_char_keyboard_H
#define __generic_char_keyboard_H

#define KEYB_IRQ 1
#define KEYB_IOPORT 0x60
#define KEYB_ACKPORT 0x61
#define KEYB_INTERFACEPORT 0x64

/* keybState defines */

#define KEYB_SELFTEST_SUCCESS 1
#define KEYB_WANTS_DATA_RESEND 3

/*
 * Macro: KEYB_BUFFERSIZE
 * description: This macro or constant is the size of the software keyboard
 * input buffer, bear in mind the hardware usually can hold a few bytes before
 * filling its buffer.
 */
#define KEYB_BUFFERSIZE 64

__uint8 *scanBuf;	// Stores scancodes.
volatile ulong scanPos = 0;
ulong readPos = 0;

/* variable: keybLedState
 * description: The current state of the keyboard LED's, it's an XOR'd value of
 * the #defines in keyboard.h
 */
__uint8 keybLedState;

/* variable: keybCtrlState
 * description: The current state of the keyboard Ctrl+Alt keys. This will be
 * used for CTRL+ALT+DEL and stuff. I can also be used for things like CTRL+C in
 * the future.
 *
 *	0 == CTRL or ALT aren't pressed
 *	1 == CTRL is pressed
 *	2 == ALT is pressed
 *	3 == CTRL+ ALT
 */
__uint8 keybCtrlState;

/* variable: keybMapMod
 * description: 0 == nothing special, normal keys..
 *		1 == shift held.
 *		2 == caps lock.
 *		3 == shift and caps lock.
 */
__uint8 keybMapMod = 0;

bool keybSelftest();
bool keybWait();
void keybSetLeds();
void keybSetLed(uchar, bool);
__uint8 keybGetLeds();
IRQ_DEF_HND(keybHandler);

/* variable: keybState
 * description: Stores wether Keyboard selftest was succesfull and wether the
 * keyboard want's data resend.
 */
__uint8 keybState;

#endif /* !__generic_char_keyboard_H */
