/*
 * mc146818.c - An MC146818 (and clones) Real Time Clock driver
 *
 * Added to the Zealos project and (C) by Mark Zealey
 * Parts (C) by Reggy
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 12/12/01
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 * 29/09/01 Mark: Generic interface added, More accurate clock-reading added.
 *	Added RTCWaitUpdate().
 */

#include <common.h>
#include <mc146818.h>
#include <io.h>

#define UPDATE_IN_PROGRESS 0x80
#define FREQ_SELECT 0xA

static inline char RTCRead(__uint8 addr)
{
	outb_p(addr, 0x70);
	return inb(0x71);
}

inline ulong RTCGet(__uint8 type)
{
	/* Special case, time wanted in YYYY format... */
	if(type == TIME_YEARS)
		return RTCGet(TIME_YEARS_LOW) + RTCGet(TIME_YEARS_HIGH) * 100;

	return BCD_TO_BIN(RTCRead(type));
}

/* Function: void RTCWaitUpdate()
 * desc: Waits until the RTC updated itself. Returns FALSE on timeout, TRUE on
 * success
 */
void RTCWaitUpdate()
{
	/* Plex has a problem with not doing this. This avoids a several-second
	 * wait in plex... */
#ifndef CONFIG_PLEX
	ulong i;

	/* Wait up to a second for the update-in-progress bit to set (Indicating
	 * an update) */
	for(i = 0; i < 1000000; i++)
		if(RTCRead(FREQ_SELECT) & UPDATE_IN_PROGRESS)
			break;

	/* Now wait up to 2.28ms for it to end the update */
	for(i = 0; i < 100000; i++)
		if(!(RTCRead(FREQ_SELECT) & UPDATE_IN_PROGRESS))
			break;
#endif /* !PLEX */
}

EXPORT_SYMBOL(RTCGet);
EXPORT_SYMBOL(RTCWaitUpdate);
