/*
 * super.c - handles reading/writing the ext2 superblock
 *
 * Added to the Zealos project and (C) by silas albrecht
 *
 * Current maintainer: silas albrecht <silas_a_a@yahoo.com>
 * Status: Orphan
 * Last update: 04/01/02
 *
 * Last CVS commit : $Id: super.c,v 1.35 2002/02/19 16:46:32 jalh Exp $
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 * well, heres a small start to ext2 support.
 *
 * Changelog:
 *
 */

#include <common.h>
#include <fs/ext2.h>
#include <fs/vfs.h>
#include <device.h>
#include <block/block.h>
#include <kmalloc.h>
#include <printk.h>
#include <stdio.h>


/* neato function to check if the block has a copy of the superblock */
bool ext2_has_sb_copy(ulong b)
{
	ulong pow3=3, pow5=5, pow7=7;

	if(b == 0 || b == 1)
		return TRUE;

	/* looks nicer than for() eh? :) */
	while (pow3 <= b || pow5 <= b || pow7 <= b) {
		if ( pow3 == b || pow5 == b || pow7 == b)
			return TRUE;
		pow3 *= 3; pow5 *= 5; pow7 *= 7;
	}

	return FALSE;
}

struct sblock *ext2_read_super(struct devNum *device)
{
	struct sblock *sb = sb_create();
	struct ext2_super_block *sb_data;
	struct ext2_sb_info *sb_info = NULL;
	int sb_loc = 2;
	int db_per_group;
	int i;
	/* Uncomment this code to make it write the ext2 magic number to the
	 * offset of the partition on the disk. Useful for making your own disk
	 * image after partitioning with fdisk - Mark
	 */
/*	char *t = kmalloc(1024);
	sb_data = (void *)t;
	kmemset(t, 0, 1024);
	sb_data->magic = EXT2_MAGICNUM;
	blk_write(device->major, device->minor, 2, 2, t);
	kfree(t);
*/
	DPRINT("Trying to read sb of device %lu:%lu\n", device->major,
			device->minor);

	sb_data = (void *)blk_read(device->major, device->minor, 2, 2);
	if(!sb_data) {
		DPRINT("couldn't read superblock?\n");
		return NULL;
	}

	if(sb_data->magic != EXT2_MAGICNUM) {
		DPRINT("Bad magic (0x%4x). Go get Harry Potter to fix.\n",
				sb_data->magic);
		goto mount_uhhohh;
	}

	/* XXX: test revision level, compatabilty.. */

        sb->blocksize_bits = sb_data->log_blk_size + 10;
	sb->blocksize = 1 << sb->blocksize_bits;

	DPRINT("blksize: %lu; bits: %u; blocks: %lu; inodes: %lu, first data "
			"block: %lu; blocks per group: %lu\n",
			sb->blocksize_bits, sb->blocksize, sb_data->blk_count,
			sb_data->ino_count, sb_data->first_data_block,
			sb_data->blks_per_group);

	/* XXX: make sure blocksize stuff is good */
	/* wheee set up a bit of sb->info */
	sb_info = kmalloc(sizeof(*sb_info));
	sb->info = sb_info;
	if (sb_data->rev_level > EXT2_REV0) {
		sb_info->ino_size = EXT2_REV0_INODE_SIZE;
		sb_info->first_ino = EXT2_REV0_FIRST_INO;
	} else {
		sb_info->ino_size = sb_data->inode_size;
		sb_info->first_ino = sb_data->first_ino;
		if (sb_info->ino_size != EXT2_REV0_INODE_SIZE) {
			DPRINT("inode size unsupported: %d\n",
					sb_info->ino_size);
			goto mount_uhhohh;
		}
	}

	DPRINT("inode size: %lu; first inode: %lu\n", sb_info->ino_size,
			sb_info->first_ino);

	sb_info->desc_per_blk = sb->blocksize /
					sizeof (struct ext2_group_desc);
	sb_info->fragsize = EXT2_MIN_FRAG_LOGSIZE << sb_data->log_frag_size;

	DPRINT("desc_per_blk: %lu; fragment size: %lu\n",
			sb_info->desc_per_blk, sb_info->fragsize);

	if(!sb_info->fragsize)
		goto mount_uhhohh;

	sb_info->frags_per_blk = sb->blocksize / sb_info->fragsize;
        sb_info->ino_per_blk = sb->blocksize / sb_info->ino_size;
	DPRINT("frags per block: %lu; inodes per block: %lu\n",
			sb_info->frags_per_blk, sb_info->ino_per_blk);

	/* XXX: add a bunch of tests to make sure we can handle this fs :) */
	sb_info->groups_count = (sb_data->blk_count - 
				sb_data->first_data_block +
				sb_data->blks_per_group - 1) / 
				sb_data->blks_per_group;

	db_per_group = sb_info->groups_count + 
			sb_info->desc_per_blk - 1 ;
	DPRINT("group-count: %lu; db_per_group: %lu\n",
			sb_info->groups_count, db_per_group);

	sb_info->gd = kmalloc (db_per_group * sizeof
						(struct ext2_group_desc*));

	if (sb_info->gd == NULL)	// out of mem?!!?
		goto mount_uhhohh;

	for (i = 0; i < db_per_group; i++) {
		/* hope this works!! :) */
		sb_info->gd[i] = (void *)blk_read(device->major, device->minor,
						sb_loc + i*2 + 2,
						sb->blocksize / BYTES_PER_SECT);
		if (sb_info->gd[i] == NULL) { // failed, so free everything.
			kfree (sb_info->gd);
			DPRINT ("err... ehh.. couldn't read gd's :(\n");
			goto mount_uhhohh;
		}
	}	

	/* XXX: check descripters!! */

	sb_info->loaded_ino_bmaps = 0;
	sb_info->loaded_blk_bmaps = 0;
	sb_info->db_per_group = db_per_group;

	sb->dev = device;
	sb->magic = EXT2_MAGICNUM;

	sb_info->drv_sb = sb_data;
	
	/* XXX: get root inode ... i bet this comment will be gone soon ;) */
	/* XXX: more setting up */

	return sb;

mount_uhhohh:
	DPRINT("Something fucked up. Exiting from ext2 for this drive\n");
	kfree(sb_info);
	kfree(sb_data);
	kfree(sb);
	return NULL;
}

/* all you need to do to get noticed :) pretty cool, eh? (silas)
 *		 -- At last you recognise my master plan! - Mark ;)
 */
FS(ext2) {
	idnum: EXT2_IDNUM,
	super_ops: {
		read_super: ext2_read_super,
	},
};

// END
