/*
 * vfs_files.c - The VFS layer for files
 *
 * Added to the Zealos project and (C) by Silas Albrecht
 *
 * Current maintainer: Silas Albrecht <silas_a_a@yahoo.com>
 * Status: Maintained
 * Last update: 12/01/02
 *
 * Last CVS commit : $Id: vfs_files.c,v 1.14 2002/02/19 16:46:32 jalh Exp $
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 *
 */

#include <common.h>
#include <vfs.h>
#include <printk.h>
#include <stdio.h>
#include <kmalloc.h>
#include <llist.h>


struct file *used_files = NULL;
struct file *free_files = NULL;
__uint32 nr_of_used = 0;	// the number of files in 'used_files'
__uint32 nr_of_free = 0;	// when a files usenum = 0, nr_of_free++ :)

/* function: struct file *get_new_file()
 * description: this function returns a new file by first checking for
 * one in free_files, if that fails it kmallocs one. then it adds
 * it to the list of files..
 */
struct file *get_new_file()
{
	struct file *to_use;
	if (nr_of_free) {
		to_use = free_files;
		dlistDel(free_files, to_use);
		nr_of_free--;
	} else { // damn .. gotta malloc one..
		to_use = (struct file *) kmalloc (sizeof(struct file));
		if (!to_use)	// uh oh..
		return NULL;
	}

	// we got one
	nr_of_used++;
	memset(to_use, 0, sizeof(struct file));
	to_use->usenum++;
	dlistPrepend(used_files, to_use);
	return to_use;
}

/* function: void put_file(struct file *file)
 * description: when a process is done with a file this is called.. if they
 * were the only ones we add it the freelist. otherwise we just dec its
 * usenum..
 */
void put_file(struct file *file)
{
	if(--file->usenum == 0) {
		dlistDel(used_files, file); /* used_files is right? - Mark */
		dlistPrepend(free_files, file);
		nr_of_free++;
		nr_of_used--;
	}
}

EXPORT_SYMBOL(get_new_file);
EXPORT_SYMBOL(put_file);
