/*
 * debug.c - Debugging routines
 *
 * Added to the Zealos project and (C) by Zwane Mwaikambo
 * Parts (C) Mark Zealey
 *
 * Current maintainer: Zwane Mwaikambo <zwane@commfireservices.com>
 * Status: Maintained
 * Last update: 21/11/01
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
*/

#include <common.h>
#include <debug.h>
#include <printk.h>
#include <stdio.h>
#include <stdarg.h>

#ifdef DEBUG

int inline __nop_function(int, char *, ...);

/* print only if the subsystem is being debugged */
#define DPRINTK(x, y...)	printk(__FILE__ ":%u (" __FUNCTION__ "): " x, (__LINE__) , ##y )

int volatile DEBUG_CPU = 0;
int volatile DEBUG_PCI = 0;
int volatile DEBUG_SMP = 0;
int volatile DEBUG_APIC = 0;
int volatile DEBUG_SUBSYS = 0;
int volatile DEBUG_FUNCTIONS = 0;	/* functions.c */
int volatile DEBUG_SPINLOCK = 0;
int volatile DEBUG_DELAY = 0;
int volatile DEBUG_IRQ = 0;
int volatile DEBUG_SIGNALS = 0;
int volatile DEBUG_PAGE = 0;
int volatile DEBUG_MM = 0;

int inline __nop_function(int a, char *f, ...)
{
	/* do absolutely nothing */
	return 0;
}

int __dbg_printk(int foo_debug, char *f, ...)
{
	char *args, out[300];

	if (!foo_debug)
		return 1;

       	vaStart(args, f);
        vsprintf(out, f, args);
	return (DPRINTK("%s", out));
}

int (*DPRINT)(int a, char *, ...) = __nop_function;

/* enable debugging for a specific subsystem */
void inline DEBUG_ON(int *foo_debug)
{
        DPRINT = __dbg_printk;
	*foo_debug = 1;
}

void inline DEBUG_OFF(int *foo_debug)
{
	*foo_debug = 0;
	DPRINT = __nop_function;
	/* Any other debugging aids you want turned off here */
}

/* drops you at a prompt, where you can view variables and perhaps set */
void inline DEBUG_PAUSE(int foo_debug)
{
	/* we might need to do a global cli to stop from being preempted
	 * by interrupt handlers */
	DPRINT = __dbg_printk;
	
	/* command line parser here */
}

/* begin debugging a code block, this is plain ugly, but i put the in_debug
 * so that you can declare local variables in debug blocks, nifty in an ugly
 * way.
 */

#define DEBUG_BEGIN(x)	if (x) { int __in_debug = 1
#define DEBUG_END	__in_debug = 0; } while(0)

EXPORT_SYMBOL(DEBUG_ON);
EXPORT_SYMBOL(DEBUG_OFF);
EXPORT_SYMBOL(DEBUG_BEGIN);
EXPORT_SYMBOL(DEBUG_END);
EXPORT_SYMBOL(DEBUG_PAUSE);
EXPORT_SYMBOL(DPRINT);

EXPORT_SYMBOL(DEBUG_PCI);
EXPORT_SYMBOL(DEBUG_SMP);
EXPORT_SYMBOL(DEBUG_APIC);
EXPORT_SYMBOL(DEBUG_SUBSYS);
EXPORT_SYMBOL(DEBUG_FUNCTIONS);
EXPORT_SYMBOL(DEBUG_SPINLOCK);
EXPORT_SYMBOL(DEBUG_DELAY);
EXPORT_SYMBOL(DEBUG_IRQ);
EXPORT_SYMBOL(DEBUG_SIGNALS);

#endif /* ifdef DEBUG */

