/*
 * kernarea.c - Functions for finding out if an address is in the kernel
 * 
 * Added to the Zealos project and (C) by Mark Zealey
 * 
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 10/07/01
 * 
 * Last CVS commit : $Id: kernarea.c,v 1.4 2002/02/19 16:46:32 jalh Exp $
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 * Description:
 * 
 * Notes/Fixme's:
 * 
 * Changelog:
 */


#include <common.h>
#include <kernarea.h>
#include <cpu.h>
#include <llist.h>
#include <kmalloc.h>

struct kernArea {
	memory start;
	memory end;
	LIST(struct kernArea);
} *addrStart = NULL;

bool inKernel(memory address)
{
	struct kernArea *t;
	listItt(addrStart, t)
		if(t->start <= address && t->end >= address)
			return TRUE;

	return FALSE;
}

errRet kernAreaAdd(memory start, memory end)
{
	struct kernArea *t = kmalloc(sizeof(struct kernArea));

	if(!t)
		return ENOMEM;

	t->start = start;
	t->end = end;

	listPrepend(addrStart, t);
	return SUCCESS;
}

errRet kernAreaDel(memory start)
{
	struct kernArea *t;

	listDel(addrStart, t, t->start == start);
	if(!t)
		return ENOEXIST;
	kfree(t);
	return SUCCESS;
}

EXPORT_SYMBOL(inKernel);
EXPORT_SYMBOL(kernAreaAdd);
EXPORT_SYMBOL(kernAreaDel);
