/*
 * printk.c - Handles printk's
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 12/12/01
 *
 * Last CVS commit : $Id: printk.c,v 1.15 2002/02/19 16:46:32 jalh Exp $
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 * This provides a means to send messages from the kernel to the monitor.
 *
 * Notes/Fixme's:
 * Maybe add a linux-like printk buffer ring?
 *
 * Changelog:
 * 09/07/01 Mark: Fixed the handling of a panic()
 * 11/07/01 Mark: Fixed an *anciant* bug where buf was always set to 0... (only
 *	noticed this after about 1 hour of debugging why when I set the first
 *	page of memory to be not in memory, plex crashed.. wierd eh :-)
 */


#include <common.h>
#include <printk.h>
#include <stdarg.h>
#include <stdio.h>
#include <syscall.h>
#include <desc.h>

/* function: errRet printk(uint level, char *fmt, ...)
 * type: global
 * description: Prints a kernel message by passing the arguments to printf() and
 * triggers some errors depending on what 'level' is it called with (KINFO etc)
 */
errRet printk(uint lev, char *fmt, ...)
{
	char *args, out[300];

	vaStart(args,fmt);
	vsprintf(out,fmt,args);

	puts(out);

	if(lev == KPANIC) {
		if(intSetup())
			syscall(dump);
		halt();
	}

	return SUCCESS;
}

EXPORT_SYMBOL(printk);
