/*
 * sched.c - The scheduler and portable routines needed for scheduling tasks
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 12/12/01
 *
 * Last CVS commit : $Id: sched.c,v 1.17 2002/02/19 16:46:32 jalh Exp $
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 * mark: Improve schedule() and tasks in general. This was done just to prove
 *	that it could be done.
 *
 * Changelog:
 * 10/06/01 mark: Redid it so it didn't use hardware-task switching.
 * 24/06/01 silas: wrote scheduler (talk about quick and dirty... i'll be
 * very angry if somebody doesn't rewrite this :)
 * 13/09/01 Zwane: added "thisBox" support, plus a few evil things ;) ...
 */

#include <common.h>
#include <sched.h>
#include <arch/kern/task.h>
#include <signals.h>
#include <cpu.h>
#include <assert.h>
#include <pit.h>
#include <irq.h>
#include <stdio.h>

extern void taskStartup();


handleIRQ(schedInt)
{
	SCHEDData->scheduler();
}

#if 1
void schedule()
{
	struct task *prev;

	__local_sti();

	// FIXME: Improve scheduling algo...
	prev = curCPU->curTask;
	curCPU->curTask = curCPU->curTask->next;
	printk(KINFO, ".");

	assert(curCPU->curTask);

	taskSwitch(prev, curCPU->curTask);
}
#else
struct task *get_best_task()
{
	struct task *task, *best = curCPU->curTask;
	listItt(curCPU->curTask, task) {
		if(task->chance > MAX_CHANCE) {
			if(task->prio) task->prio--;	// heh heh
			return task;
		}
		task->chance += task->prio;
		if(task->chance <= 0)
			task->prio++;
		if(task->chance > best->chance)
			best = task;
	}
	return best;
}

void schedule()
{
	struct task *prev;

	__local_sti();

	// give the others a chance
	curCPU->curTask->chance -= curCPU->curTask->prio * SCHED_KICKBACK;
	prev = curCPU->curTask;

	curCPU->curTask = get_best_task();
	assert(curCPU->curTask);

	if(prev != curCPU->curTask)
		taskSwitch(prev, curCPU->curTask);
}
#endif

initFunc schedInit()
{
	curCPU->curTask = NULL;
	SCHEDData = kmalloc(sizeof(SCHEDData));
	if (!SCHEDData) {
		printk(KINFO,"SCHEDULER: Unable to kmalloc %d bytes! (OOM?)\n");
		return ENOMEM;
	}

	thisBox->SCHEDData = SCHEDData;
	SCHEDData->scheduler = schedule;

	taskStartup();

	addHandler(PIT_IRQ, schedInt, NO_DELAY, NULL);
//	if(sigConNum(, schedInt, 1, 255, NULL) != SUCCESS)
//		return EOHHBUGG;

//	IRQAdd(PIT_IRQ);

//	schedule();

	return SUCCESS;
}

void schedCleanup(void)
{
	printk("SCHEDULER: Removing scheduler at 0x%x...", SCHEDData->scheduler);
	
	/* hmm... */
	delHandler(schedInt);

	kfree(SCHEDData);
	thisBox->SCHEDData = NULL;

	printk("Done\n");
}

EXPORT_SYMBOL(schedule);
