/*
 * start.c - The place where it all begins... :)
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 12/12/01
 *
 * Last CVS commit : $Id: start.c,v 1.56 2002/02/19 16:46:32 jalh Exp $
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 * OK, so this is the file where the true ZealOS begins. startPoint starts it
 * off, then it just loads up some basic mods (precompiled in) and loads the
 * other mods. pretty simple eh? :)
 *
 * Notes/Fixme's:
 *
 * Changelog:
 */

// FLARP! - silas

#include <common.h>
// Place other .h files to include here...
#include <stdio.h>
#include <desc.h>
#include <irq.h>
//#include <block/floppy.h>
#include <signals.h>
#include <modules.h>
#include <functions.h>
#include <video.h>
#include <init.h>
#include <kmalloc.h>
#include <pit.h>
#include <io.h>
#include <block/ata.h>
#include <video.h>
#include <cpu.h>
#include <printk.h>
#include <device.h>
#include <fs/partition.h>
#include <sched.h>
#include <char/keyboard.h>
#include <fs/vfs.h>
#include <syscall.h>
#include <gdb-stub.h>
#include <kern/time.h>
#include <pit.h>
#include <math64.h>
#include <pci.h>
#include <smp.h>
#include <subsystems.h>

initFunc printBanner();

/* variable: initFunc (*startFuncs[])(void)
 * description: This is a list of functions of the initFunc function() type.
 * In startPoint, it simply has to itterate thru these to initalize various
 * parts of the module loader, rather than writing a list of them below. This
 * means that you can add a function easier.
 */
/* the reason we can't have these be auto-detected (like filesystems for
 * instance is because we'd have to preserve the order - silas
 */
initFunc (*startFuncs[])(void) = {
	videoInit,
	printBanner,
	initInit,
	memInit,
	subsysInit,
	funcInit,
	GDTInit,
	signalsInit,
	IRQInit,
	IDTInit,
#ifdef CONFIG_GDB
	GDBInit,
#endif
#ifdef __SMP__
	SMPInit,
#endif /* __SMP__ */
	swapInit,
	CPUInit,
	deviceInit,
	syscallInit,
#ifdef CONFIG_TIME
        timeInit,
#endif

#ifdef CONFIG_TASK
	schedInit,
#endif

#ifdef CONFIG_PCI
	PCIInit,
#endif /* PCI */

#ifdef CONFIG_KEYBOARD
	keyboardInit,
#endif

//#ifdef CONFIG_FLOPPY
//	floppyInit,
//#endif

#ifdef CONFIG_ATA
	ATAInit,
#endif
#ifdef CONFIG_PARTITION
	partitionInit,
#endif
#ifdef CONFIG_VFS
	vfsInit,
#endif

	0			// Null-terminated list
};

// First thing to be printed
initFunc __preInitFunc printBanner()
{
	printk(KINFO, "ZealOS module loader ver " KERN_MAJOR_STR "."
			KERN_MINOR_STR "." KERN_REL_MAJOR_STR "."
			KERN_REL_MINOR_STR " (%u) compiled " COMPILE_TIME " by "
			COMPILE_USER "@" COMPILE_HOST " on a " ARCH_STR " with "
			COMPILE_VER "\n", CURRENT_KERN_VER);
	return SUCCESS;
}

SIG_DEF_HND(hang)
{
#ifdef CONFIG_MM_STATS
	dump_stats();
#endif
	if(subsysIsLoaded("time")) {
		ulong denom = Hz;
		struct timeval curtime;
		__uint32 t = mathDiv64(get_jiffies64(), &denom);

		printk(KINFO, "Zealos Uptime has been %lu second%s\n", t,
				(t == 1)?"":"s");
		printk(KINFO, "Number of jiffies: %lu\n", get_jiffies64());
		gettimeofday(&curtime);
		printk(KINFO, "Time is %lu and %lu milliseconds\n",
				curtime.tv_sec, curtime.tv_usec / 1000);
		/* This is just for delay loop caliberation and gettimeofday()
		 * testing. - Mark */
/*		delay(10000);
		gettimeofday(&curtime);
		printk(KINFO, "Time is %lu and %lu milliseconds\n",
					curtime.tv_sec, curtime.tv_usec / 1000);
*/	}

	printk(KINFO, "\nEnd of ZealOS reached, hanging\n");
}

/* function: void startPoint()
 * description: This is basically the place where all the different arch's jump
 * to after they've done whatever. This function basically just calls off a list
 * of functions in startFuncs[]. After all this is done, it just hangs there...
 * ;-)
 */
void startPoint()
{
	ulong j;

	// Cycle thru the init functions :)
	for(j=0; startFuncs[j]; j++)
		if(startFuncs[j]() != SUCCESS)
			printk(KINFO, "Function 0x%p (%lu) reported failure\n",
					startFuncs[j], j);

	// Last thing to be called.. :)
	sigCon("halt", hang, 255, NO_DELAY, NULL);
	sigTrig("shutdown", NULL);
	sigTrig("halt", NULL);

	halt();

#ifdef DEBUG
	printk(KERROR, "Shouldn't get here...\n");
	while(1);		// Last fall-back
#endif /* DEBUG */
}
