/*
 * vmalloc.c - A Virtual memory allocation system
 * 
 * Added to the Zealos project and (C) by Mark Zealey
 * 
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 12/12/01
 * 
 * Last CVS commit : $Id: vmalloc.c,v 1.5 2002/02/19 16:46:32 jalh Exp $
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 * Description:
 * 
 * Notes/Fixme's:
 * 
 * Changelog:
 */


#include <common.h>
#include <vmalloc.h>
#include <page.h>
#include <mem.h>
#include <assert.h>
#include <kmalloc.h>

void *vmalloc(ulong len, uint options)
{
	long start;
	ulong i;

	DPRINT("len: %lu, options: %x\n", len, options);

	start = pageGetBlock(NULL, len, PAGE_RW | PAGE_KERN);
	if(start == -1)
		return NULL;

	i = len;
	while(len--) {
		void *p = kmallocPageO(options);

		if(pageSet(NULL, start + len, PAGE_ADDRESS((memory)p) |
					PAGE_RW | PAGE_PRESENT | PAGE_KERN)
				!= SUCCESS) {
			DPRINT("pageSet() failed, len = %lu, options %lu, p %p",
					len, options, p);
			return NULL;
		}
	}

	pageReload();

	while(i--) {
		*(long*)((start + i) * PAGE_SIZE) = 0x12345678;
	}

	DPRINT("Gave address %lx\n", start * PAGE_SIZE);

	return (void *)(start * PAGE_SIZE);
}

errRet vfree(void *mem, ulong len)
{
	ulong start = (ulong)mem / PAGE_SIZE;
	ulong i;

	assert(mem);

	for(i = 0; i < len; i++) {
		struct pgTblEnt *pg = pageLookup(NULL, start + i);
		kfreePage((void *)PAGE_ADDRESS(pg->entry));
	}

	return pageFreeBlock(NULL, len, start);
}

EXPORT_SYMBOL(vmalloc);
EXPORT_SYMBOL(vfree);
