/*
 * dma.c - Do DMA stuff
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Orphan
 * Last update: 27/07/00
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 */

#include <common.h>
#include <io.h>

char page[4] = { 0x87, 0x83, 0x81, 0x82};

#define MASK(x) (((x)<4)?0xA:0xD4)
#define MODE(x) (((x)<4)?0xB:0xD6)
#define CLEAR(x) (((x)<4)?0xC:0xD8)
#define PAGE(x) (((x)<4)?page[x]:page[x-4]+8)
#define ADDRESS(x) (((x)<4)?(x)*2:((x)*2)+0xC0)
#define COUNT(x) (ADDRESS((x))+1)

void channelToMemory(uchar channel, ulong address, uint length, bool read)
{
	outb(4 | channel, MASK(channel));		// Disable channel
	outb(0x44+channel+((read)?4:0), MODE(channel));	// Set the mode
	outb(address & 0xFF, ADDRESS(channel));		// Set the low address
	outb(address & 0xFF00, ADDRESS(channel));	// Set the high address
	outb(address >> 16, PAGE(channel));		// Set the page
	outb(length & 0xFF, COUNT(channel));		// Set the low length
	outb(length & 0xFF00, COUNT(channel));		// Set the high length
	outb(channel, MASK(channel));			// Enable channel
}

EXPORT_SYMBOL(channelToMemory);
