/*
 * task.c - A task implimentation for i386
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 12/12/01
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 * mark: Just look thru the file... :)
 *
 * Changelog:
 */

#include <common.h>
#include <task.h>
#include <printk.h>
#include <desc.h>
#include <cpu.h>
#include <stdio.h>
#include <syscall.h>
#include <kmalloc.h>
#include <init.h>


extern void firstForkReturn();		// See desc.c

void idle()
{
	asm ("1: rep; nop\n jmp 1b;");
	// while(1) /* do nothing */ ;
}

void fastCall __taskSwitch(struct task *prev, struct task *next)
{
	curCPU->arch.TSS.esp0 = (__uint32)next->stack;

	getSeg("fs", prev->arch.fs);
	getSeg("gs", prev->arch.gs);
	loadSeg("fs", next->arch.fs);
	loadSeg("gs", next->arch.gs);
}

errRet kernThread(void (*func)(), void *arg)
{
	volatile ulong t;

	t = getStackPointer();
	printk(KINFO, "%p %p %lu\n", curCPU->curTask->stackBase, getStackPointer(), (memory)curCPU->curTask->stackBase - getStackPointer());
	syscall(clone);
	WRITE_BARRIER();
	printk(KINFO, "%p %p %lu\n", curCPU->curTask->stackBase, getStackPointer(), (memory)curCPU->curTask->stackBase - getStackPointer());
	printk(KINFO, "%p\n", func);
	if(t == getStackPointer())	// Parent thread
		return SUCCESS;

	while(1);
	(*func)();
}

//	asm volatile(
			/* Save stack */
//			"movl %%esp, %1		\n"
			/* clone */
//			"int $" STR(SYSCALL_INT) "	\n"
			/* Check to see if we are daddy or not */
//			"cmpl %%esp, %1			\n"
			/* If we are, carry on */
//			"je 1f				\n"
//			"popl %%eax			\n"	// ???
			/* Pass reg in both eax and on the stack */
//			"movl %4, %%eax			\n"
//			"pushl %%eax			\n"
			/* Call it!! */
//			"call *%3			\n"
			/* Maybe mop up ? */
//			"2: jmp 2b			\n"
			/* Escape place */
/*			"1:				\n"
			: "=a" (ret), "=S" (t)
			: "a" (__NR_clone), "b" (func), "r" (arg)
			: "memory");
	return ret;
*/
syscallFunc(clone)
{
	struct task *t, *cur = curCPU->curTask;
	void *s;
	ulong stackLen;
	struct allRegs *r;

	t = kmalloc(sizeof(struct task));
	t->eip = (ulong) firstForkReturn;

	stackLen = (memory)cur->stackBase - (memory)regs;
	s = kmallocPage();
	t->stackLen = PAGE_SIZE;
	t->stackBase = s + t->stackLen;
	t->stack = t->stackBase - stackLen;
	kmemcopy(t->stack, regs, stackLen);
	r = t->stack;
	r->eip = regs->eip;

	printk(KINFO, "stack base: %p; stack len: %lu; stack: %p; regs: %p; cur base: "
			"%p\n",
			t->stackBase, stackLen, t->stack, regs, cur->stackBase);

	t->prio = 5;

	t->next = curCPU->curTask->next;
	curCPU->curTask->next = t;

	return SUCCESS;
}

void taskStartup()
{
	struct task *t;
	kmemset(&curCPU->arch.TSS, 0, sizeof(struct TSS));
	curCPU->arch.TSSGDTEntry = GDTAddEntry(&curCPU->arch.TSS,
				sizeof(struct TSS), FALSE, FALSE, 9, 0, FALSE);

	ltr(curCPU->arch.TSSGDTEntry);

	t = kmalloc(sizeof(struct task));
//	curCPU->curTask->prio = 30;
	t->next = t;
	t->stackBase = initVals.stackStart;
	curCPU->curTask = t;

	kernThread(idle, NULL);
}
