/*
 * block.h - A simple interface to block-devices
 *
 * Added to the Zealos project and (C) by Silas Albrecht
 *
 * Current maintainer: Silas Albrecht <silas_a_a@yahoo.com>
 * Status: Maintained
 * Last update: 26/09/01
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 */

#ifndef __block_H
#define __block_H

#include <device.h>
#include <kmalloc.h>
#include <math64.h>
#include <cpu.h>
#include <printk.h>

#define BYTES_PER_SECT		512


struct blk_cmd {
	__uint32 start;		// In SECTORS!
	__uint32 length;	// In SECTORS!
	deviceCmd type;
	void *data;
};

struct block_geom {
	ulong cylinders;
	ushort heads;
	ulong sectors;
	ulong size;		// Size in sectors
};

static inline errRet blk_cmd(major major, minor minor, __uint32 start,
				__uint32 length, deviceCmd type, char *buf)
{
	struct devNum device;
	struct blk_cmd cmd;

	device.major = major;
	device.minor = minor;

	cmd.start = start;
	cmd.length = length;
	cmd.data = buf;
	cmd.type = type;

	return devCall(&device, &cmd);
}

static inline char *blk_read(major major, minor minor, __uint32 start,
				__uint32 length)
{
	char *buf;

	buf = kmalloc(length * BYTES_PER_SECT);
	if(!buf)
		return NULL;

	if(blk_cmd(major, minor, start, length, DEV_READ, buf) != SUCCESS ) {
		kfree(buf);
		return NULL;
	}

	return buf;
}

#define blk_write(maj, min, s, l, buf) blk_cmd(maj, min, s, l, DEV_WRITE, buf)

#endif
