/*
 * keyboard.h - Some global definitons for the keyboard driver
 *
 * Added to the Zealos project and (C) by Daniel Abraham
 * Also (C) by Reggy
 *
 * Current maintainer: Reggy <reggy@ekky.org>
 * Status: Maintained
 * Last update: 04/01/02
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 * Keyboard functions implemented by 
 *
 * Notes/Fixme's:
 *
 * Changelog:
 */

#ifndef __keyboard_H
#define __keyboard_H

initFunc keyboardInit();
int kgetc();
char *kgets(char *buf);
bool kkbhit();

#define ESC	27
#define FBUT	58
#define HOME	71
#define UP	72
#define PGUP	73
#define LEFT	75
#define RGHT	77
#define END	79
#define DOWN	80
#define INS	81
#define PGDN	82

/*
 * Macro: SCROLL_LOCK, NUM_LOCK, CAPS_LOCK, ALT, CTRL, SHIFT
 * Description: This is (or will be) used internally as a bitmask
 * to represent which keys are depressed when a key is pressed
 * or released.
 */
#define SCROLL_LOCK 1
#define NUM_LOCK 2
#define CAPS_LOCK 4
#define ALT 8
#define CTRL 16
#define SHIFT 32

/*
 * Macro: LED_NUM, LED_CAPS, LED_SCROLL
 * Description: These are the valid values that can be passed to
 * the keybSetLeds(char, bool) function. The first argument can
 * be set to any of these constants below and the second argument
 * can be set to either TRUE (1) or FALSE (0).
 */
#define LED_NUM (1 << 0)
#define LED_CAPS (1 << 1)
#define LED_SCROLL (1 << 2)

#endif // __keyboard_H
