/*
 * ext2.h - Header file for ext2 fs implimentation
 *
 * Added to the Zealos project and (C) by Silas Albrecht
 *
 * Current maintainer: Silas Albrecht <silas_a_a@yahoo.com>
 * Status: Maintained
 * Last update: 04/01/02
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 */

#ifndef __ext2_H
#define __ext2_H

#include <device.h>
#include <fs/vfs.h>
#include <printk.h>

#define EXT2_MAGICNUM	0xEF53

#define EXT2_REV0		0
#define EXT2_REV0_FIRST_INO	11
#define EXT2_REV0_INODE_SIZE	128

#define EXT2_MINBLK_LOGSIZE	10
#define EXT2_MIN_BLK_SIZE	(1 << EXT2_MINBLK_LOGSIZE)
#define EXT2_MAX_BLK_SIZE	4096

#define EXT2_MIN_FRAG_LOGSIZE	10
#define EXT2_MIN_FRAG_SIZE	(1 << EXT2_MIN_FRAG_LOGSIZE)
#define EXT2_MAX_FRAG_SIZE	4096

/* fs states */
#define EXT2_CLEAN	0x0001
#define EXT2_ERROR	0x0002

/* used in its fs_type struct.. */
#define EXT2_IDNUM	0x83

/* Special inodes numbers */
#define EXT2_BAD_INO		1	/* Bad blocks */
#define EXT2_ROOT_INO		2	/* Root */
#define EXT2_ACL_IDX_INO	3	/* ACL */
#define EXT2_ACL_DATA_INO	4	/* ACL */
#define EXT2_BOOT_LOADER_INO	5	/* Boot loader */
#define EXT2_UNDEL_DIR_INO	6	/* Undelete directory */

/* ext2 fs block group descriptor */
struct ext2_group_desc {
	__uint32 blk_bitmap;	/* blocks bitmap */
	__uint32 ino_bitmap;	/* inodes bitmap */
	__uint32 ino_tables;	/* inodes table  */
	__uint16 num_blk_free;
	__uint16 num_ino_free;
	__uint16 num_dirs;
	__uint16 unused;
	__uint32 reserved[3];
};

/* super block!!!  ugh.. :) */
struct ext2_super_block {
	__uint32 ino_count;
	__uint32 blk_count;
	__uint32 reserved_blk_count;
	__uint32 free_blk_count;

	__uint32 free_ino_count;
	__uint32 first_data_block;
	__uint32 log_blk_size;		/* Block size */
	__uint32 log_frag_size;		/* Fragment size */

	__uint32 blks_per_group;
	__uint32 frags_per_group;
	__uint32 inos_per_group;

	__uint32 mtime;			/* Mount time */
	__uint32 wtime;			/* Write time */

	__uint16 mnt_count;		/* Mount count */
	__uint16 max_mnt_count;		/* Maximal mount count */
	__uint16 magic;
	__uint16 fs_state;
	__uint16 errors;		/* Behaviour when detecting errors */
	__uint16 minor_rev_level;

	__uint32 lastcheck;
	__uint32 fsck_checkintv;	/* max. time between checks */
	__uint32 creator_os;
	__uint32 rev_level;
	__uint16 def_resuid;		/* Default uid for reserved blocks */
	__uint16 def_resgid;		/* Default gid for reserved blocks */
	__uint32 first_ino;		/* First non-reserved inode */
	__uint16 inode_size;		/* size of inode structure */
	__uint16 block_group_nr;	/* block group # of this superblock */
	__uint32 feature_compat;	/* compatible feature set */
	__uint32 feature_incompat;	/* incompatible feature set */
	__uint32 feature_ro_compat;	/* readonly-compatible feature set */
	__uint32 unused[230];		/* blank until end of block.. */
} __PACKED__;				// ack! for testing..

#define EXT2_MAX_GROUP	8

// contains misc in memory information for a superblock
struct ext2_sb_info {
	struct ext2_super_block *drv_sb;
	struct ext2_group_desc **gd;	/* group descriptors */
	__uint32 ino_per_blk;
	__uint32 fragsize;
	__uint32 frags_per_blk;
	__uint32 db_per_group;		/* descripter groups per block */
	__uint32 groups_count;
	__uint32 ino_size;
	__uint32 first_ino;
	__uint32 desc_per_blk;
	__uint16 loaded_ino_bmaps;
	__uint16 loaded_blk_bmaps;
};

#endif
