/*
 * vfs.h - VFS header file
 *
 * Added to the Zealos project and (C) by Silas Albrecht
 *
 * Current maintainer: Silas Albrecht <silas_a_a@yahoo.com>
 * Status: Maintained
 * Last update: 04/01/02
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 */

#ifndef __vfs_H
#define __vfs_H

#include <device.h>

// generic mount flags.. (or flag for now ;)
#define MNT_RDONLY	1

/*
 * since we don't even have a fs.. i didn't bother putting in a lot of
 * variables so this is pretty empty.....
 */
struct inode {  // instead of having random inode lists floating around
		// we just have the sb point to its list..

	ulong		i_number;
	ulong		usenum;
	/* file permissions and stuff here.. */
	/* owner */
	/* group */
	/* size in bytes */
	/* last access */

	struct i_ops	*inode_ops;
	ulong		blocks;
	ushort		flags;
	bool		locked;
	bool		dirty;

	struct sblock	*sb;

	struct inode	*prev;
	struct inode	*next;
};

/* 
 * if you've read linux vfs source code you may notice this is 
 * pretty similar to their vfsmnt and superblocks structs..
 * i got rid of the vfsmnt struct ... all it's used for is multiple
 * mount points.. i say tough bananas, this is easier.. make a symlink :P
 *		- silas
 */
struct sblock {
	struct devNum 	*dev;
	char		*dirname;	// where its mounted...
	char		*devname;	// such as /dev/hda2
	ulong		blocksize;
	uchar		blocksize_bits;
	uchar		lock;
	bool		read_only;	// can't this go in flags?
	uchar		dirty;		// change indicator
	struct fs_type	*type;
	struct s_ops	*super_ops;	// pointer to type->super_ops
	ulong		flags;		// mounting flags
	ulong		magic;		// magic num..			

	struct inode	*dirty_inodes;	// used for quick sync'ing (err..)
	struct inode	*inodes;

	struct sblock	*next;
	struct sblock	*prev;

	void		*info;		// Pointer to some info (fs-specific)
};

// here's another structure...
struct file {
	__uint32	mode;
	__uint16	flags;
	__uint16	usenum;		// in use count..
	// owner and stuff here.... later.. :)
	struct f_ops	*file_ops;
	struct inode	*inode;
	struct file	*next;
	struct file	*prev;
};

struct f_ops {
	// read.. write lseek ... read for now ;)
	int (*read) (struct inode *, struct file *, char *, int);
};

struct i_ops {
// 	struct file_operations * default_file_ops;
	int (*create) (struct inode *,const char *,int,int,struct inode **);
	int (*lookup) (struct inode *,const char *,int,struct inode **);
	int (*link) (struct inode *,struct inode *,const char *,int);
	int (*unlink) (struct inode *,const char *,int);
	int (*symlink) (struct inode *,const char *,int,const char *);
	int (*mkdir) (struct inode *,const char *,int,int);
	int (*rmdir) (struct inode *,const char *,int);
	int (*mknod) (struct inode *,const char *,int,int,int);
	int (*rename) (struct inode *,const char *,int,struct inode *,const char *,int, int);
	int (*readlink) (struct inode *,char *,int);
	int (*follow_link) (struct inode *,struct inode *,int,int,struct inode **);
//	int (*readpage) (struct inode *, struct page *); ermm...
//	int (*writepage) (struct inode *, struct page *);
	int (*bmap) (struct inode *,int);
	void (*truncate) (struct inode *);
	int (*permission) (struct inode *, int);
	int (*smap) (struct inode *,int);
};

// shamelessly stolen from linux (ssfl =)
struct s_ops {
	void (*read_inode) (struct inode *);
//	int (*notify_change) (struct inode *, struct iattr *);
	void (*write_inode) (struct inode *);
	void (*put_inode) (struct inode *);
	void (*put_super) (struct sblock *);
	struct sblock *(*read_super)(struct devNum *);
	void (*write_super) (struct sblock *);
//	void (*statfs) (struct sblock *, struct statfs *, int);
	int (*remount_fs) (struct sblock *, int *, char *);
};

struct fs_type {
	__uint8 idnum;
	struct s_ops super_ops;
	struct fs_type *next;
	struct fs_type *prev;
};

/* Use this to pass info about the fs to the vfs layer */
#define FS(fs) struct fs_type __SECTION__(".fsStructs") __##fs##Struct = 

////// please, PLEASE .. if you add a function add it here too :)

// vfs_super.c
errRet fs_type_reg (struct fs_type *);
errRet fs_type_unreg (struct fs_type *);
struct sblock *sb_create();
struct sblock *sb_find(struct devNum *);
errRet sb_reg (struct sblock *);
errRet sb_unreg (struct devNum *);
initFunc vfsInit();

// vfs_inode.c
struct inode *inode_lookup(struct inode *, ulong);
inline void inode_sync_one(struct inode *);
void inode_sync_list(struct inode *);
struct inode *iget (struct sblock *sb, __uint16 i_num);
void iput (struct inode *i);

// vfs_files.c
struct file *get_new_file();
void put_file(struct file *file);

// vfs_cache.c
void cache_marks_dirty(major, minor, __uint32, __uint32, char *);
char *cache_read(major, minor, __uint32, __uint32);
inline errRet cache_write(major, minor, __uint32, __uint32, char *);
void cache_write_dev(major, minor);
void cache_write_all();

/* macro: cache_write(major major, minor minor, __uint32 start, __uint32, length, char *data)
 * description: writes data to disk if its cache node is dirty or it doesn't
 * have a cache node..
 */

	
// that wasn't much fun.. (silas)
// Is it meant to be (yes :P -silas) (sorry for the kluges, I don't see how 
// we're meant to fix them tho :/ - Mark

#endif
