/*
 * assert.h - Helpful debugging macros
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 04/01/02
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 * 23/12/00 arjan: Fixed ASSERT with the do{...}while(0)
 */

#ifndef __ASSERT_H
#define __ASSERT_H

#include <printk.h>

/* Function: assert(expr)
 * arg0: The condition to test
 * description: Tests expr, if expr is false, it calls assertError() with a
 * level of KINFO. Only does anything if DEBUG is defined.
 */
#define assert(exp) ASSERT(KINFO,(exp))

/* Function: ASSERT(level, expr)
 * arg0: Level to fail with (KINFO, KPANIC, etc...)
 * arg1: The condition to test
 * description: Tests expr, if expr is false it calls assertError() with the
 * level specified in level. Used by assert() with a level of KINFO. Only does
 * anything if kernel is configured with DEBUG.
 */
#ifdef DEBUG
#define ASSERT(lev,exp) do {if(!(exp)) assertError(lev,#exp,__FILE__,__LINE__);} while (0)

void assertError(char *, char *, char *, uint);

#else
#define ASSERT(lev,exp)
#endif

#endif /* __ASSERT_H */
