/*
 * byte.h - Generic byte operations
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 15/09/01
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 */

#ifndef __generic_byte_H
#define __generic_byte_H

/* macro: swap16 swap32 swap64 (d)
 * desc: Swaps the bytes order in 16/32/64 bits of data.
 */
#define __swap16(d) ((__uint16)(					\
		(((__uint16)(d) & (__uint16)0xFF) << 8) |		\
		(((__uint16)(d) & (__uint16)0xFF00) >> 8) ))
#define __swap32(d) ((__uint32)(					\
		(((__uint32)(d) & (__uint32)0xFF) << 24) |		\
		(((__uint32)(d) & (__uint32)0xFF00) << 8) |		\
		(((__uint32)(d) & (__uint32)0xFF0000) >> 8) |		\
		(((__uint32)(d) & (__uint32)0xFF000000) >> 24) ))
#define __swap64(d) ((__uint64)(					\
		(((__uint64)(d) & (__uint64)0xFF) << 56) |		\
		(((__uint64)(d) & (__uint64)0xFF00) << 40) |		\
		(((__uint64)(d) & (__uint64)0xFF0000) << 24) |		\
		(((__uint64)(d) & (__uint64)0xFF000000) << 8) |		\
		(((__uint64)(d) & (__uint64)0xFF00000000) >> 8) |	\
		(((__uint64)(d) & (__uint64)0xFF0000000000) >> 24) |	\
		(((__uint64)(d) & (__uint64)0xFF000000000000) >> 40) |	\
		(((__uint64)(d) & (__uint64)0xFF00000000000000) >> 56) ))

#define swap_macro(bit) static inline __uint##bit __vswap##bit(__uint##bit d) \
	{								\
		return __swap##bit(d);					\
	}
swap_macro(16)
swap_macro(32)
swap_macro(64)
#undef swap_macro

// We optomize for constants
#define swap16(d) ((__builtin_constant_p((__uint16)(d)))? __swap16(d) :	\
		__vswap16(d))

/* macro: beToLe16 beToLe32 beToLe64(data)
 * desc: Convert from 16/32/64 bits of big endian data to the same amount of
 * bits in little endian format.
 */
#define beToLe16(d) swap16(d)
#define beToLe32(d) swap32(d)
#define beToLe64(d) swap64(d)

/* macro: leToBe16 leToBe32 leToBe64(data)
 * desc: Convert from 16/32/64 bits of little endian data to the same amount of
 * bits in big endian format.
 */
#define leToBe16(d) swap16(d)
#define leToBe32(d) swap32(d)
#define leToBe64(d) swap64(d)

/* macro: beToCPU16 beToCPU32 beToCPU64 leToCPU16 leToCPU32 leToCPU64 (data)
 * desc: Convert from 16/32/64 bits of big/little endian data to the type of
 * the local CPU
 */
#if BYTE_ORDER == 0			// Little endian box
#define beToCPU16(d) beToLe16(d)
#define beToCPU32(d) beToLe32(d)
#define beToCPU64(d) beToLe64(d)

#define leToCPU16(d) (d)
#define leToCPU32(d) (d)
#define leToCPU64(d) (d)
#else					// Big endian box
#define leToCPU16(d) leToBe16(d)
#define leToCPU32(d) leToBe32(d)
#define leToCPU64(d) leToBe64(d)

#define beToCPU16(d) (d)
#define beToCPU32(d) (d)
#define beToCPU64(d) (d)
#endif

// *cough*
#define CPUToLe16(d) leToCPU16(d)
#define CPUToLe32(d) leToCPU32(d)
#define CPUToLe64(d) leToCPU64(d)
#define CPUToBe16(d) beToCPU16(d)
#define CPUToBe32(d) beToCPU32(d)
#define CPUToBe64(d) beToCPU64(d)

#endif
