/*
 * debug.h - Debugging definitions
 *
 * Added to the Zealos project and (C) by Zwane Mwaikambo
 * Parts (C) Mark Zealey
 *
 * Current maintainer: Zwane Mwaikambo <zwane@commfireservices.com>
 * Status: Maintained
 * Last update: 12/12/01
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 * 14/11/01	Zwane:  First implemented
 * 18/11/01	Zwane:	More sinning...
 * 19/11/01	Zwane:  WIP
 * 11/12/01 Mark: removed it as it doesn't work so well yet ;>
 */

#ifndef __DEBUG_H
#define __DEBUG_H

#define OPT_IS_COMPILED	(1 << 0)		/* Option is to be compiled */
#define OPT_IS_DEBUG	(1 << 1)		/* With debugging? */
#define OPT_IS_MODULE	(1 << 2)		/* As a module? */
/* Add more here? Also, add to tools/make.pl too */

/* Only do this stuff if we're debugging */
#ifdef DEBUG

# include <printk.h>

/* macro: DPRINT(config, printk-like args...)
 * desc: Prints the specified string in a special debug format, if the config
 * for 'config' is marked as debug. See also, DPRINT().
 */
# define MDPRINT(a, b, c...) do {					\
		if((a) & OPT_IS_DEBUG)					\
			printk(KDEBUG, __FILE__ ":%u: " b, __LINE__, ##c); \
	} while(0)

# if CURRENT & OPT_IS_DEBUG
/* macro: DEBUG_INIT()
 * desc: Initalize the debugging in the current file. To actually do some
 * debugging, see DEBUG_START().
 */
#  define DEBUG_INIT() bool __debugged = FALSE

/* macro: DEBUG_START()
 * desc: Start debugging the current file's code (IE, print stuff that was
 * specified to print via DDPRINT()). You must DEBUG_INIT() the file first.
 */
#  define DEBUG_START() __debugged = TRUE

/* macro: DEBUG_END()
 * desc: Stop debugging the current file, as started by DEBUG_START().
 */
#  define DEBUG_END() __debugged = FALSE

/* macro: DPRINT(printk-like args...)
 * desc: Prints the specified string in a special debug format, if the current
 * module is marked as debug. See also, MDPRINT()
 */
#  define DPRINT(a, b...) MDPRINT(CURRENT, a, ##b)

/* macro: DDPRINT(printk-like args...)
 * desc: Prints the specified string, like DPRINT(), but only if the current
 * file is marked as debug (via DEBUG_START).
 */
#  define DDPRINT(a, b...) do { if(__debugged) DPRINT(a, ##b); } while(0)

# endif /* CURRENT & OPT_IS_DEBUG */

#endif /* DEBUG */


/* 'padding' functions */

#ifndef DEBUG_INIT
#define DEBUG_INIT() do {} while(0)
#endif /* !DEBUG_INIT */

#ifndef DEBUG_START
#define DEBUG_START() do {} while(0)
#endif /* !DEBUG_START */

#ifndef DEBUG_END
#define DEBUG_END() do {} while(0)
#endif /* !DEBUG_END */

#ifndef DPRINT
#define DPRINT(a...) do {} while(0)
#endif /* !DPRINT */

#ifndef DDPRINT
#define DDPRINT(a...) do {} while(0)
#endif /* !DDPRINT */

#ifndef MDPRINT
#define MDPRINT(a...) do {} while(0)
#endif /* !MDPRINT */

#endif /* !__DEBUG_H */
