/*
 * device.h - Header file for the devices implimentation
 *
 * Added to the Zealos project and (C) by Silas Albrecht
 *
 * Current maintainer: Silas Albrecht <silas_a_a@yahoo.com>
 * Status: Maintained
 * Last update: 13/01/02
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 */

#ifndef __device_H
#define __device_H

#include <majors.h>

typedef __uint32 major;
typedef __uint32 minor;

typedef enum {
	DEV_READ,
	DEV_WRITE,
	DEV_SEEK,
	DEV_RESET,
	DEV_DOOR_LOCK,
	DEV_LOCK_DOOR,
	DEV_UNLOCK_DOOR,
	DEV_DETECT_PARAMS,
	DEV_GET_GEOM,
} deviceCmd;

struct devNum {
	major major;
	minor minor;
};

typedef errRet (devHandler)(struct devNum *, void *);

initFunc deviceInit();
errRet devReg(struct devNum *, devHandler *);
errRet devDelAll(struct devNum *, bool);
errRet devCall(struct devNum *, void *);
bool devExists(struct devNum *device);

/* function: errRet devDel(struct devNum *device)
 * description: Calls off devDelAll() with a 'delAll' value of FALSE.
 */
#define devDel(t) devDelAll(t, FALSE)

/* function: deviceHnd(func)
 * description: Creates a subroutine called 'func' that gets data passed to it,
 * and is of the type errRet. The data passed to it is 'struct devNum *num' and
 * 'void *data', which can be used to get arguments passed etc. See also,
 * signalHnd().
 */
#define deviceHnd(func)						\
errRet func(struct devNum *num, void *data);			\
EXPORT_SYMBOL(func);						\
errRet func(struct devNum *num, void *data)

#endif // __device_H
