/*
 * errors.h - Standard error definitions
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 03/11/01
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 */

#ifndef __errors_H
#define __errors_H

/* typedef: errRet
 * description: The standard return type of a function to indicate failure or
 * success etc...
 */
typedef __uint8 errRet;

/* typedef: initFunc
 * description: Return type for a function called to to init a certian module.
 * Could in the future be mapped to a neat macro for module initilization and
 * loading etc...
 */
typedef errRet initFunc;

/* macro: SUCCESS
 * description: The standard return of errRet type functions.
 */
#define SUCCESS 0

/* macro: EOHHBUGG
 * description: An unknown error (would have been EBUGGER but the function
 * didn't finish ;). This is the general 'Dazed and confused that something
 * fucked up' return :) */
#define EOHHBUGG 1

/* macro: ENOMEM
 * description: Out of memory. */
#define ENOMEM 2

/* macro: ETOOBIG
 * description: Something exceeded a bounds test. */
#define ETOOBIG 3

/* macro: ELOCK
 * description: A lock request failed. */
#define ELOCK 4

/* macro: ENOSIG
 * description: Can't find the specified signal name. */
#define ENOSIG 5

/* macro: ENOEXIST
 * description: Something doesn't exist. */
#define ENOEXIST 6

/* macro: ETIMEOUT
 * description: Timeout. */
#define ETIMEOUT 7

/* macro: EPROC
 * description: Processor error. */
#define EPROC 8

/* macro: EINVALID
 * description: Something invalid happened. */
#define EINVALID 9

/* macro: EEXIST
 * description: The thing already exists... */
#define EEXIST 10

/* macro: EABORT
 * description: The command had to be aborted. */
#define EABORT 11

/* macro: ENODEP
 * description: Special macro that says that a required dependancy was not
 * located that was needed to use this function.
 */
#define ENODEP 12

#endif // __errors_H
