/*
 * irq.h - Generic IRQ interface
 * 
 * Added to the Zealos project and (C) by Mark Zealey
 * 
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 04/01/02
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 * Description:
 * 
 * Notes/Fixme's:
 * 
 * Changelog:
 */

#ifndef __generic_kern_irq_H
#define __generic_kern_irq_H

#include <desc.h>
#include <signals.h>
#include <arch/kern/irq.h>

initFunc IRQInit();
errRet IRQAdd(ulong);
errRet addHandler(ulong, sigHandler *, ulong, void *);
errRet delHandler(ulong, sigHandler *);

// Definitions done in the arch-specific stuff
ulong IRQInitArch();
void EOIIntr(ulong);
errRet IRQMask(ulong, bool);

struct IRQHandler {
	ulong things;			// Number of things attached to this IRQ
};

handleInt(triggerIRQ);

typedef struct {
	archIRQData_t arch;
} IRQData_t;

extern IRQData_t *IRQData;		// See irq.c
extern ulong IRQNum;

/* macro: IRQAddr(IRQ)
 * arg0: IRQ number
 * description: Converts an IRQ to an offset in the IDT.
 */
#define IRQAddr(IRQ) ((IRQ) + IRQ_OFFSET)

struct IRQ {
	uint num;			/* IRQ Number */
	struct allRegs *regs;
};

/* macro: IRQ_HND(name, data arg, trigger arg name)
 * arg0: Function name
 * arg1: The name and type of the data passed to it, 'void *dataBlk' for
 * example.
 * arg2: the name to give the struct IRQ that is passed. for example 'irq'.
 *
 * desc: Creates an IRQ handler that will be called off via the signals code. It
 * basically does the same as SIG_HND() but the trigger data is replaced with a
 * struct IRQ.
 */
#define IRQ_HND(x, y, z) SIG_HND(x, y, struct IRQ *z)

/* macro: IRQ_DEF_HND(name)
 * desc: This calls IRQ_HND with 'name', 'void *dataBlk' and 'irq' as the
 * arguments.
 */
#define IRQ_DEF_HND(x) IRQ_HND(x, void *dataBlk, irq)

#endif /* __generic_kern_irq_H */
