/*
 * kmalloc.h - Header file for memory management
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 04/01/02
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 */

#ifndef __KMALLOC_H
#define __KMALLOC_H

/* macro: PAGE_SIZE
 * description: This contains the definition of the page size of the current
 * arch. It doesn't really mean much until paging etc is done, but I've set it
 * to 4Kb (the i386 page size) so it will be easier to do paging.
 */
#define PAGE_SIZE (1 << PAGE_SIZE_BITS)

// Options to kmalloc etc
#define OPT_DMA		(1 << 0)
#define OPT_NORMAL	(1 << 1)

#define kmallocPageO(o) kmallocPages(1, o)
#define kmallocPage() kmallocPageO(OPT_DMA | OPT_NORMAL)
#define kmallocDMAPage() kmallocPageO(OPT_DMA)
#define kmallocZeroPage() ({						\
	void *page;							\
	page = kmallocPage();						\
	kmemset(page, 0, PAGE_SIZE);					\
	page;								\
})

#define kfreePage(a) kfreePages(a, 1)

#ifdef CONFIG_MM_STATS
void dump_stats();
#endif

bool isMemory(ulong);
void *kmalloc(ulong);
memory kgetSize(void *);
void kfree(void *);
void *krealloc(void *, ulong);
initFunc memInit();
initFunc swapInit();
errRet kfreePages(void *p, uint num);
void *kmallocPages(uint num, uint options);

#include <debug.h>

#if CONFIG_MM & OPT_IS_DEBUG
extern bool mmDebug;
#endif

#endif /* !__KMALLOC_H */
