/*
 * stdarg.h - Library for extracting arguments for functions with
 *	variable-length args
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 07/02/02
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 * A set of macros for retrieving arguments from functions with variable
 * argument lists, for example int foo(char *bar,...).
 *
 * Notes/Fixme's:
 *
 * Changelog:
 */

#ifndef __STDARG_H
#define __STDARG_H

#define va_tmp(a) (((sizeof(a)+sizeof(int)-1)/sizeof(int))*sizeof(int))

/* function: va_start(pointer, first_arg)
 * arg0: A pointer that will be passed to vaArg().
 * arg1: The first argument that was passed to the function
 * description: Needed to initalize 'pointer' before calling va_arg to get the
 * arguments passed to a foo(bar, ...)-type function.
 */
#define va_start(a,b) a=((char *)&(b)+va_tmp(&(b)))

/* function: va_arg(pointer, type)
 * description: Returns the argument passed to the function where you
 * va_start()'d 'pointer'. Type is some type that can be sizeof()'d, IE int,
 * void *, struct foo etc..
 */
#define va_arg(a,b) (a+=va_tmp(b),*((b *)(a-va_tmp(b))))

#endif // __STDARG_H
