/*
 * subsystems.h - Sub-system tracking fuctions
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 04/01/02
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 */

#ifndef __generic_kern_subsystems_H
#define __generic_kern_subsystems_H

initFunc subsysInit();
errRet subsysAdd(char *, void *, errRet (*)(), errRet (*)());
errRet subsysDepAdd(char *, char *);
errRet subsysDepDel(char *, char *);
errRet subsysDel(char *, bool);
bool subsysIsLoaded(char *);
errRet subsysGetData(char *, void **);

struct subsystem {
	char *name;
	void *ptr;
	errRet (*init)();
	errRet (*exit)();
};

/* macro: SUBSYSTEM(name, pointer, start_func, stop_func)
 * desc: Used to declare a sub-system if you are initalized before the
 * subsystems code is.
 */
#define SUBSYSTEM(n,p,s,e)						\
char __subStr_##n[] __SECTION__(".subStrs") = #n;			\
struct subsystem __sub_##n __SECTION__(".subSyms")			\
	= { __subStr_##n, p, s, e }

/* macro: REQUIRES_SUBSYSTEM(thisSubsystem, requires)
 * desc: This will add the dependancy of 'thisSubsystem' to the subsystem
 * 'requires'. If 'requires' is not currently registered with the code, it will
 * make the current function return ENODEP.
 */
#define REQUIRES_SUBSYSTEM(t,n)						\
		do { if(subsysDepAdd(t,n) != SUCCESS) return ENODEP; } while(0)

/* macro: SUBSYSTEM_ON(name, paramInfo, start_func, stop_func)
 * desc: Tells the subsystem code that the subsystem 'name' is now up and
 * running. Registers a pointer to a param block, NULL otherwise. If operation
 * is not successful, it will make the current function return EOHHBUGG.
 */
#define SUBSYSTEM_ON(n,p,s,e) do {					\
		if(subsysAdd(n,p,s,e) != SUCCESS) return EOHHBUGG;	\
	} while(0)

#define SUBSYSTEM_OFF(n) subsysDel(n, FALSE)

#endif /* __generic_kern_subsystems_H */
