/*
 * time.h - Generic time functions
 *
 * Added to the Zealos project and (C) by Reggy
 * Parts (C) by Mark Zealey <mark@itsolve.co.uk>
 *
 * Current maintainer: Reggy Ekkebus <reggy@ekky.org>
 * Status: Maintained
 * Last update: 04/01/02
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 */

#ifndef __generic_kern_time_H
#define __generic_kern_time_H

#include <spinlock.h>

/* We must hold this lock to write or read jiffies64. (on a 32-bit box...) */
extern spinlock_t jiffies_lock;
extern __uint64 jiffies64;

struct timespec {
        ulong tv_sec;		/* Seconds since epoch */
        ulong tv_msec;		/* Miliseconds */
};

struct timeval {
	long tv_sec;		/* Seconds */
	long tv_usec;		/* Microseconds */
};

initFunc timeInit();
errRet gettimeofday(struct timeval *ts);
ulong mktime(uint sec, uint min, uint hour, uint day, uint month, uint year);

/* Macro: __uint64 get_jiffies64()
 * desc: This returns the current number of jiffies. Do not try and access
 * jiffies without holding a lock as it is 64-bit, and thus on some systems it
 * can't be done atomically.
 */
#if ARCH_BITS < 64
#define get_jiffies64() ({						\
		__uint64 cur_jif;					\
		ulong flags;						\
		spinlockIRQSave(&jiffies_lock, &flags);			\
		cur_jif = jiffies64;					\
		spinlockIRQRestore(&jiffies_lock, &flags);		\
		cur_jif;						\
	})
#else /* ARCH_BITS < 64 */
/* Can be done atomically */
#define get_jiffies64() get_jiffies()
#endif /* ARCH_BITS < 64 */

/* macro: ulong get_jiffies()
 * desc: Returns the current number of jiffies (atomically).
 */
#if BYTE_ORDER == 0 || ARCH_BITS > 32
#define get_jiffies() (((volatile ulong *)&jiffies64)[0])
#else /* BYTE_ORDER == 0 || ARCH_BITS > 32 */
#define get_jiffies() (((volatile ulong *)&jiffies64)[1])
#endif /* BYTE_ORDER == 0 || ARCH_BITS > 32 */

#endif /* __generic_kern_time_H */
