/*
 * mm/mem.h - internal MM defines etc
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 12/12/01
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 */

#ifndef __MEM_H
#define __MEM_H

#include <llist.h>
#include <arch/mm/page.h>

/* macro: PAGE_ADDRESS(address)
 * desc: Returns the address rounded down the the page that it's in.
 */
#define PAGE_ADDRESS(x) ((x) & ~(PAGE_SIZE - 1))

errRet slabInit();
errRet pageTableInit();
errRet pageInit();
errRet swapInit();
uint pageGetOrder(ulong size);

#ifdef CONFIG_MM_POISON
#define KMALLOC_POISON 0xA5
#define KFREE_POISON 0x5A
#endif

#define MAX_ORDER 5

// Page options
#define PAGE_USED	(1 << 0)
#define PAGE_UNUSED	(0 << 0)

typedef struct page {
	memory page;		// Page number (address / PAGE_SIZE)
	uint options;		// Page options
	int refs;		// Number of things ref'ing this page
	DLIST(struct page);	// Link to next page in the list
} page;

/* struct: memArea
 * desc: This struct contains all the info needed for a specific region of
 * memory.
 */
typedef struct {
	page *free[MAX_ORDER];	// List of free pages in this memory area
//	uint cost;		// Relative 'cost' of accessing this memory
//	uint pressure;		// Pressure of the current memory place
	ulong pagesFree;	// Free pages in this area
	ulong pagesTotal;	// Number of pages used from this area
} memArea;

extern memArea memZone[MAX_AREAS];

extern bool swapSetup;

#ifdef CONFIG_MM_STATS

/* struct: stats
 * description: This struct is a private struct for the purposes of mm
 * debugging. If MM_STATS is defined, it keeps all the memory stats in
 * this struct.
 */
struct stats {
	ulong kmallocs;		// Number of kmalloc's ever
	ulong kfrees;		// Number of kfree's ever
	ulong buddyTables;	// number of buddy table's in use
	ulong kmallocCurrent;	// Number currently kmalloc'd
	ulong pagesUsed;	// Pages that have been kmalloc'd ever
	ulong pagesFree;	// Pages that have been kfree'd ever
	ulong pagesCurrent;	// current number of pages in use
};

extern struct stats stats;

#endif /* CONFIG_MM_STATS */

errRet pageFree(page *p, uint order);
page *pageAlloc(uint order, uint options);

#endif /* !__MEM_H */
