/*
 * arch_common.h - Arch-specific defines etc
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 01/11/01
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 * This file is for i386 specific global defines. for something not this
 * specific, please use include/generic/kern/common.h. Thanks - Mark
 *
 * Notes/Fixme's:
 *
 * Changelog:
 */

#ifndef __ARCH_COMMON_H
#define __ARCH_COMMON_H

/* macro: LOCK
 * description: If we're on a SMP sys, we have to prefix the atomic operation
 * with 'lock' so noone else fucks with us while we're doing it. Sound pretty
 * normal? stupid intel...
 */
#ifdef __SMP__
#define LOCK "lock;"
#else
#define LOCK ""
#endif

typedef char __int8;
typedef short __int16;
typedef long __int32;
typedef long long __int64;

typedef unsigned char __uint8;
typedef unsigned short __uint16;
typedef unsigned long __uint32;
typedef unsigned long long __uint64;

// What you cast a void * to to be able to read it 'normally'
typedef ulong memory;

/* macro: MAX_IDT_ENTRIES
 * description: The maximum number of IDT entries allowed on the specific
 * platform.
 */
#define MAX_IDT_ENTRIES 0x100

#define BYTE_ORDER 0		// We're on a little-endian box

#define ARCH_BITS 32		// 32-bit arch :)

/* macro: BREAKPOINT()
 * desc: This causes a breakpoint to happen in the kernel. Usially used while
 * remote-gdb'ing.
 */
#define BREAKPOINT() asm("int3");

#if PROCESSOR >= 5
#define HAVE_CYCLE_COUNT
#else /* PROCESSOR >= 5 */
#undef HAVE_CYCLE_COUNT
#endif /* PROCESSOR >= 5 */

#endif	// __ARCH_COMMON_H
