/*
 * irq.h - IRQ's for i386
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 13/09/01
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 * 13/09/01 Zwane Added a few defines to aid in coding, added "thisBox" support
 */

#ifndef __irq_H
#define __irq_H

#include <spinlock.h>

/* macro: IRQ_OFFSET
 * description: The beginning of the interrupt area that the IRQ's are mapped
 * onto. Used by many things including IRQAddr().
 */
#define IRQ_OFFSET 0x30			// NOTE: must be a mutiple of 8.

// Number of IRQ's we have
#define IRQ_NUM 16
/* select PIC number depending on which IRQ */
#define PIC_NUM(x)	(((x) < 7) ? 0 : 1)
#define ISR_REG		0x0B
#define IRR_REG		0x0A

typedef struct {
	uchar masks[IRQ_NUM/8];
	/* we don't want to do a IRQSave/Restore, outp_b, inp_b each time! */
	uchar cached_reg;
} archIRQData_t;

#endif // __irq_h

#include <generic/kern/irq.h>
