/*
 * math64.h - i386-specific 64-bit maths routines
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 14/10/01
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 * 30/06/01 Mark: Fixed some things and neatened the stuff up a bit
 */

#ifndef __math_64bit_H
#define __math_64bit_H

/* function: to64(high, low, result)
 * description: Converts a 32-bit 'high' and 'low' to a 64-bit 'result'.
 */
#define to64(h,l,r) asm("" : "=A" (r) : "a" (l), "d" (h));

/* function: from64(number, high, low)
 * description: Converts a 64-bit number 'number' into 2 32-bit numbers ('high'
 * and 'low')
 */
#define from64(n,h,l) asm("" : "=a" (l), "=d" (h) : "A" (n));

/* function: __uint32 mathDiv64(__uint64 num, __uint32 *denom)
 * description: Divides the 64-bit number in 'num' by 'denom' and returns the
 * remainder in 'denom' and the result.
 */
static inline __uint64 mathDiv64(__uint64 num, __uint32 *denom)
{
	__uint64 ret;
	__uint32 a, b, c = 0;

	from64(num, a, b);

	/* Prevent overflows */
	if(a) {
		c = a / *denom;
		a %= *denom;
	}

	asm("divl %4"
			: "=a" (b), "=d" (*denom)
			: "0" (b), "1" (a), "r" (*denom));

	to64(c, b, ret);

	return ret;
}

/* function: __uint64 mathMul64(__uint32 numA, __uint32 numB)
 * desc: Multiplys the 2 input operands and returns the result.
 */
static inline __uint64 mathMul64(__uint32 numA, __uint32 numB)
{
	__uint64 ret;
	__uint32 a,b;

	asm("mull %0"
			: "=d" (a), "=a" (b)
			: "0" (numA), "1" (numB));

	to64(a,b,ret);
	return ret;
}

#define do_mathn64(a,b)							\
static inline __uint64 math##a##64(__uint64 *numA, __uint64 numB)	\
{									\
	__uint64 ret;							\
	__uint32 a,b,c,d;						\
	from64(numA, a, b);						\
	from64(numB, c, d);						\
	asm(	#a "l %5, %3;"						\
		#b "l %4, %2"						\
		: "=a" (a), "=b" (b)					\
		: "0" (a), "1" (b), "c" (c), "d" (d)			\
		: "cc" );						\
	to64(a,b,ret);							\
	return ret;							\
}

/* function: __uint64 mathAdd64(__uint64 numA, __uint64 numB);
 * description: Adds the 64-bit number in 'numA' to the 64-bit number in 'numB'
 * and returns the result.
 */
do_mathn64(Add, adc)

/* function: __uint64 mathSub64(__uint64 numA, __uint64 numB);
 * description: Subtracts the 64-bit number in 'numA' from the 64-bit number in
 * 'numB' and returns the result.
 */
do_mathn64(Sub, sbb)
#undef do_mathn64

#endif // __math_64bit_H
