/*
 * pci.h - i386 PCI stuff
 *
 * Added to the Zealos project and (C) by Zwane Mwaikambo
 * Parts (C) by Mark Zealey <mark@itsolve.co.uk>
 *
 * Current maintainer: Zwane Mwaikambo <zwane@commfireservices.com>
 * Status: Maintained
 * Last update: 04/01/02
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 * 29/09/01 Mark: Minor fixes more to come.
 */

#include <cpu.h>

#ifndef __i386_pci_H
#define __i386_pci_H

#define BIOS32_MAGIC		REVERSE_PACK('_', '3', '2', '_')
#define BIOS32_BEGIN_SEARCH	0xE0000
#define BIOS32_END_SEARCH	(0x100000 - 0x10)
/* PCI signature: "PCI " */
#define PCI_SIGNATURE		REVERSE_PACK('P', 'C', 'I', ' ')
/* PCI service signature: "$PCI" */
#define PCI_SERVICE		REVERSE_PACK('$', 'P', 'C', 'I')
#define PCI_CONF_ADDR		0xCF8
#define PCI_CONF_DATA		0xCFC
#define PCI_PCIBIOS_PRESENT	0xB101
/*
 * Format of BIOS32 Service Directory header structure:
 * Offset  Size    Description     (Table F0021)
 * 00h  4 BYTEs   signature "_32_" == 0x5F32335F
 * 04h    DWORD   physical address of BSD entry point (see #F0022)
 * 08h    BYTE    header structure version number (currently 00h)
 * 09h    BYTE    header structure length in paragraphs (currently 01h)
 * 0Ah    BYTE    checksum (8-bit sum of all bytes in structure,
 *               including this one, should equal zero)
 * 0Bh  5 BYTEs   reserved (0)
 *
 */

typedef struct {
	__uint32 signature;
	__uint32 entry;
	uchar version;
	uchar length;
	uchar csum;
	uchar reserved[5];
} __PACKED__ bios32_t;

typedef struct {
	unsigned reg: 8,
		 func:3,
 		 dev:5,
		 bus:8,
		 reserved:7,
		 enable:1;
} __PACKED__ PCIConfigAddr_t;

typedef struct {
	__uint16 vendor_id;
	__uint16 device_id;
	__uint16 cmd;
	__uint16 status;
	__uint8 rev_id;
	__uint8 interface;
	__uint8 sub_class;
	__uint8 base_class;
	__uint8 cache_line_size;
	__uint8 latency_timer;
	__uint8 header_type;
	__uint8 bist;
} __PACKED__ PCIConfig_t;

#define PCI_MULTIFUNCTION(dev) ((dev)->header_type & 0x80)
#define PCI_EXTCFG_TYPE(dev) ((dev)->header_type & 0x7F)

#define PCI_EXTCFG_NORMAL	0
#define PCI_EXTCFG_PCI_PCI	1
#define PCI_EXTCFG_CARDBUS	2

/* if PCI_EXTCFG_TYPE == NORMAL, then the following applies: */
typedef struct {
	__uint32 base[6];
	__uint32 cardbus_ptr;
	__uint16 subsystem_vendor_id;
	__uint16 subsystem_id;
	__uint32 expansion_rom_ptr;
	__uint8 capabilities_ptr;
	__uint8 res1[7];
	__uint8 irq_line;
	__uint8 irq_pin;
	__uint8 max_gnt;
	__uint8 max_lat;
} __PACKED__ PCIConfigExtNormal_t;

/* Data struct for use with subsystems, please also check with
 * this struct for box functionality before trying any stunts
 * in your code, e.g. if hasBios32 is false then don't use it.
 */
typedef struct {
	struct ljmpStr bios32Entry;
	struct ljmpStr PCIBiosIndirect;
	bios32_t *bios32;
	__uint32 PCIBiosEntry;
	__uint8 last_bus;
	__uint8 major_ver;
	__uint8 minor_ver;
	__uint8 access_type;	/* direct (config 1/2) or bios */
} __PACKED__ PCIData_t;

/* Some external structures by which we return our information */
#include <llist.h>

struct PCIBus {
	struct PCIDevice *devs;
	__uint8 num_devs;
	LIST(struct PCIBus);
};

struct PCIDevice {
	__uint16 vendor;
	__uint16 device;
	__uint8 class;
	__uint8 subclass;
	__uint8 revision;
	LIST(struct PCIDevice);
};

initFunc PCIInit();

#include <generic/kern/pci.h>

#endif /* !__i386_pci_H */
