/*
 * task.h - i386-specific multi-tasking routines
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 08/09/01
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 */

#ifndef __task_H
#define __task_H

#define BMP_PORT_LEN 0 //32

// Only used for it's io bitmap protection stuff, rest is done in sw
struct TSS {
	__uint16 back, __bh;
	__uint32 esp0;
	__uint16 ss0, __ss0h;
	__uint32 esp1;
	__uint16 ss1, __ss1h;
	__uint32 esp2;
	__uint16 ss2, __ss2h;
	__uint32 cr3;
	__uint32 eip;
	__uint32 eflags;
	__uint32 eax;
	__uint32 ecx;
	__uint32 edx;
	__uint32 ebx;
	__uint32 esp;
	__uint32 ebp;
	__uint32 esi;
	__uint32 edi;
	__uint16 es, __esh;
	__uint16 cs, __csh;
	__uint16 ss, __ssh;
	__uint16 ds, __dsh;
	__uint16 fs, __fsh;
	__uint16 gs, __gsh;
	__uint16 ldt, __ldth;
	__uint16 trace, bmp;
	__uint8 ioBmp[BMP_PORT_LEN+1];		// +1 is for 0xFF last entry
} __PACKED__;

struct LDT {
	__uint16 offset;		// Offset in GDT
	void *start;
	ulong length;
};

struct archTask {
	struct LDT LDT;
	__uint16 fs, gs;
};

struct task;

extern void fastCall __taskSwitch(struct task *, struct task *);

// Clever, we must swap esp *before* we call __taskSwitch to be able to get our
// return address back... - Mark
#define taskSwitch(p,n)							\
	asm volatile(							\
		".globl firstForkReturn		\n"			\
		/* We now save the prev esp and load the new one */	\
		"movl %%esp, %0			\n"			\
		"movl %2, %%esp			\n"			\
		/* Save the old eip so it won't infinatly loop here */	\
		"movl $1f, %1			\n"			\
		/* Now fake the return so it will go straight to the
		 * place where it's needed (call __taskSwitch == push
		 * $1f; jmp __taskSwitch; 1:) This fakes the return
		 * address to be the entry to the place we left off last
		 * time :-) */						\
		"pushl %3			\n"			\
		"jmp __taskSwitch		\n"			\
		"1:				\n"			\
		: "=m" (p->stack), "=m" (p->eip)			\
		: "m" (n->stack), "m" (n->eip),				\
		/* Because we're using fastCall, args are passed in eax
		 * and edx... - Mark */					\
			"a" (p), "d" (n), "b" (p))

#include <sched.h>

#endif
