/*
 * page.h - Paging implimentation for the i386
 * 
 * Added to the Zealos project and (C) by Mark Zealey
 * 
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 06/08/01
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 * Description:
 * 
 * Notes/Fixme's:
 * 
 * Changelog:
 */

#ifndef __i386_kern_page_H
#define __i386_kern_page_H

#include <bitmap.h>

/* macro: PAGE_SIZE_BITS
 * description: PAGE_SIZE = 1 << PAGE_SIZE_BITS; needed for gcc 2.7 backwards
 * compatability (64-bit optomization probs)
 */
#define PAGE_SIZE_BITS 12

// Standard intel paging bits
#define PAGE_DIRTY		(1 << 6)
#define PAGE_ACCESSED		(1 << 5)
#define PAGE_USER		(1 << 2)
#define PAGE_KERN		(0 << 2)
#define PAGE_RW			(1 << 1)
#define PAGE_RO			(0 << 1)
#define PAGE_PRESENT		(1 << 0)
#define PAGE_SWAPPED		(0 << 0)

// Different memory areas
#define AREA_NORMAL 0
#define AREA_DMA 1
#define MAX_AREAS 2

struct pgTblEnt {
	ulong entry;
};

struct pgDirEnt {
	ulong entry;
};

typedef struct {
	struct pgDirEnt *dir;
	uint pageDirSize;
	ulong pageTblSize;
	bitmap freeSlots;
} pageDir;

uchar pageZone(memory page);
long pageGetBlock(pageDir *tbl, ulong length, uint opts);
errRet pageFreeBlock(pageDir *tbl, ulong length, long pos);
errRet pageSet(pageDir *tbl, memory page, ulong options);
errRet pageDel(pageDir *tbl, memory page, ulong options);
errRet pageAdd(pageDir *tbl, memory page, ulong options);
struct pgTblEnt *pageLookup(pageDir *tbl, memory page);

#define pageReload() asm volatile ("movl %%cr3, %%eax; movl %%eax, %%cr3" ::: "eax")

#endif /* __i386_kern_page_H */
