/*
 * atomic.h - Atomic operations on x86-64
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 30/06/01
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 * Atomic operations for the x86-64, used in locking etc...
 *
 * Notes/Fixme's:
 *
 * Changelog:
 */

#ifndef __atomic_H
#define __atomic_H

/* typedef: atomic
 * description: A integer for use with atomic functions such as atomicInc()
 */
typedef volatile __int64 atomic;

#define ATOMIC_INIT 0

/* macro: atomic(atomicNum)
 * arg0: An argument of the type 'atomic'
 * description: this returns a interger so you can set a type of 'atomic' to
 * some value to start off with. EG atomic t; atomic(t) = 5; printk("%d",
 * atomic(t)).
 */
#define atomic(x) (x)

/* function: atomicAdd(atomicNum, number)
 * arg0: An argument of the type 'atomic'
 * arg1: A number to add to it
 * description: Atomically adds 'number' onto the atomic variable 'atomicNum'.
 */
#define atomicAdd(x, y) asm volatile ("addq %0, %1" : "=m" atomic(x) : \
		"g" (y), "m" atomic(x))

/* function: atomicSub(atomicNum, number)
 * arg0: An argument of the type 'atomic'
 * arg1: A number to subtract from it
 * description: Atomically subtracts 'number' from the atomic variable
 * 'atomicNum'.
 */
#define atomicSub(x, y) asm volatile ("subq %0, %1" : "=m" atomic(x) : \
		"g" (y), "m" atomic(x))

/* function: void atomicInc(atomic *atomicNum)
 * arg0: An argument of the type 'atomic'
 * description: Increments 'atomicNum' by 1 in an atomic operation.
 */
static inline void atomicInc(atomic *x)
{
	asm volatile("incq %0" : "=m" atomic(*x) : "m" atomic(*x));
}

/* function: void atomicDec(atomic *atomicNum)
 * arg0: An argument of the type 'atomic'
 * description: Decrements 'atomicNum' by 1 in an atomic operation.
 */
static inline void atomicDec(atomic *x)
{
	asm volatile("decl %0" : "=m" atomic(*x) : "m" atomic(*x));
}

#endif // __atomic_H
