/*
 * init.h - x86-64 specific initalization routines
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 04/01/02
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 */

#ifndef __init_H
#define __init_H

errRet parseLilo();
initFunc initInit();

/* macro: MAX_COMMAND_LEN
 * description: Maximum length of command line that we parse from lilo
 */
#define MAX_COMMAND_LEN 128

/* macro: ARG_AREA
 * description: Address that the bootloader arguments are put at
 */
#define ARG_AREA 0x9F000

/* macro: MMAP_LEN
 * description: Maximum length of the memory map that we get (E820 command)
 */
#define MMAP_LEN 0x100

/* macro: MMAP_ENTRIES
 * description: Maximum memory map entries we can have. See also: MMAP_LEN
 */
#define MMAP_ENTRIES (MMAP_LEN/sizeof(struct memMap))

/* struct: bloaderVals
 * description: The struct that we get the values out bootloader passed to us
 * from. (see also boot/boot.inc)
 */
struct bloaderVals {
	ulong kernSize;
	ulong memSize;
	char *liloStr;
	char *mmapStr;
	ulong mmapLen;
	ulong kernStart;
	void *symsStart;
	ulong symsLen;
	void *delableStart;
	ulong delableLen;
	void *fsStart;
	ulong fsLen;
	void *syscallStart;
	ulong syscallLen;
	void *stackStart;
};

/* struct: memMap
 * description: This is a struct that we rip as much info out of the bios as
 * possible from. It's basically what the BIOS gives us as info, however only
 * newer BIOS's actually give us this, so for older ones we have to make it up
 * as we go along ;) The meaning of each 'type' entry is shown in the table
 * below:
 * 
 * value | meaning<BR>
 * ------+-----------------------------------<BR>
 * 1     | normal memory, avaliable to us!!<BR>
 * 2     | reserved (eg ROM, mmapped device)<BR>
 * 3     | ACPI Reclaim memory<BR>
 * 4     | ACPI NVS memory (dont touch!!)<BR>
 */
struct memMap {
	__uint64 base;	// Base addr (in bytes)
	__uint64 len;	// Length (in bytes)
	__uint32 type;	// Type, see above
} __PACKED__;

/* struct: basicValues
 * description: The basic values that we've got from the bootloader
 */
struct basicValues {
	char commandLine[MAX_COMMAND_LEN];	// The un-parsed command line
	ulong kernelLength;	// The length of the kernel (in bytes)
	memory kernelStart,	// The start point of the kernel (in bytes)
	       reservedBeg;	// The place where the reserved area begins
	void *symsStart;
	ulong symsLen;
	void *delableStart;
	ulong delableLen;
	void *fsStart;
	ulong fsLen;
	void *syscallStart;
	ulong syscallLen;
	void *stackStart;
};

extern struct basicValues initVals;
extern struct memMap mminfo[MMAP_ENTRIES];

#endif // __init_H
