/*
 * lock.h - locking mechanisms for x86-64
 * 
 * Added to the Zealos project and (C) by Mark Zealey
 * 
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 30/06/01
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 * Description:
 * 
 * Notes/Fixme's:
 * 
 * Changelog:
 */

#ifndef __i386_kern_lock_H
#define __i386_kern_lock_H

#include <atomic.h>
#include <cpu.h>

typedef struct {
	atomic count;
} lock_t;

#define LOCK_INIT {0}

#ifndef __SMP__
extern atomic in_lock;		/* defined in cpu.c */
#endif

static inline void lockInit(lock_t *lock)
{
#ifdef __SMP__
	atomic(lock->count) = 0;
#endif
}

static inline void __local_lock(lock_t *lock)
{
	if(atomic(in_lock) == 0)	// At top-level
		__local_cli();
	atomicInc(&in_lock);
}

static inline void __local_unlock(lock_t *lock)
{
	atomicDec(&in_lock);
	if(atomic(in_lock) == 0)	// At top-level
		__local_sti();
}

static inline void lock(lock_t *lock)
{
#ifdef __SMP__
	while(lock->count)
		/* Wait*/;

	atomicInc(&lock->count);
#else
	__local_lock(lock);
#endif
}

static inline void unlock(lock_t *lock)
{
#ifdef __SMP__
	atomicDec(&lock->count);
#else
	__local_unlock(lock);
#endif
}

#endif /* __i386_kern_lock_H */
