package Doc;
use strict;
use Carp;
require 'tools/Parse.pm';
use File::Find;

my $def_scope = 'local';

# Returns all the documentation about the specified module
sub get_docu {
	my %ret;

	# Get module files
	my %config = &Parse::Config('config');
	my %mod_depend = Parse::Mod_deps($config{ARCH},'mod_depend');
	my %t = Parse::Mod_deps('generic','mod_depend');

	push @{$mod_depend{$_}}, @{$t{$_}} for keys %t;

	for my $mod (@_) {
		next unless exists $mod_depend{$mod};

		for my $file (@{$mod_depend{$mod}}) {
			parse($file,\%ret)
		}
	}
	return %ret;
}

sub parse {
	my ($file, $hash) = @_;
	
	open F, $file or croak "Can't open file $file: $!";

	# Get the comments out and parse block-by-block
	while(@_ = &get_comment_block(\*F)) {
		my (%stuff, $crap);
		my $last = \$crap;
		for(@_) {
			s/<BR>/\n/;		# HACK!!
			if(/^(function|macro):\s*([^(]+)(.*)?/i) {
				$stuff{type} = lc $1;
				my $t = $2;
				my $v = $3;
#####				$t =~ s/\s+//g; # JALH!!! :) -silas
				$stuff{name} = join "\n", split /,/, $t;
				$stuff{calling_list} = $v;
				if($stuff{calling_list} =~ /\)$/) {
					$last = \$crap;
				} else {
					$last = \$stuff{calling_list};
				}
				$stuff{scope} = $def_scope unless 
							exists $stuff{scope};
			} elsif(/^(struct|variable|typedef)(?:ure)?:\s*(.+)/i) {
				$stuff{type} = lc $1;
				$stuff{name} = join "\n", split /\s*,\s*/, $2;
				$stuff{scope} = $def_scope unless 
							exists $stuff{scope};
				$last = \$stuff{name};
			} elsif(/^type:\s*(.+)/i) {
				$stuff{scope} = lc $1;
				$last = \$stuff{scope};
			} elsif(/^arg(\d+(?:\.\.\d+)?):\s*(.*)/i) {
				my ($a,$b) = ($1,$2);
				$a =~ s/\.\./ to /;
				$stuff{args}{$a} = $b;
				$last = \$stuff{args}{$a};
			} elsif(/^comment:\s*(.+)/i) {
				$stuff{name} = $1;
				$last = \$stuff{name};
			} elsif(/^desc(?:ription)?:\s*(.*)/i) {
				$stuff{description} = $1;
				$last = \$stuff{description};
			} else {
				$$last .= " $_";	# Continued line
			}
		}

		if(keys %stuff and $stuff{name}) {
			my $func = $stuff{name};
			delete $stuff{name};
			chomp $stuff{$_} for grep {$_ ne 'args'} keys %stuff;
			$hash->{$file}{$func} = { %stuff } if keys %stuff;
		}
	}

	close F;
}

sub get_comment_block {
	my ($fh, $incb, $inc, @comments, $cur, $t) = (shift,0,0);

	while(<$fh>) {
		chomp;

		my $t = $inc;
		$cur = $_ if $incb;

		if(m!/\*!) {
			$cur = $';
			$incb = $inc = 1;
		} elsif(m!\*/!) {
			$cur = $`;
			$incb = 0;
			$inc = 1;	# Still in the comment
		} elsif(m!//!) {
			$cur = $';
			$incb = 0;
			$inc = 1;
		} elsif(!$incb) {
			$cur = '';
			$inc = 0;
		}

		$cur =~ s/^[\s*]*//;
		
		if(defined $cur) {
			if($cur) {
				push @comments, $cur;
			} elsif($inc) {
				push @comments, "\n";
			}
		}
		last if $t and !$inc;  # End of this comment block
	}

	return @comments if wantarray;
	return join "\n", @comments;
}
