#!/usr/bin/perl -w
# Last update: 27/06/01
use POSIX qw/strftime/;
use Text::Wrap;

my %hash = (
	name => 'Mark Zealey',
	email => 'mark@itsolve.co.uk',
	time => strftime('%H:%M:%S', localtime),
	date => strftime('%d/%m/%Y', localtime),
);

################################################################################
my $inc = 'template';
my $endc = <<EOF;
 */

#include <common.h>
#include <\$file.h>

EOF

my $endh = <<EOF;
 */

#ifndef __\$arch_\$dir_\$file_H
#define __\$arch_\$dir_\$file_H

#endif /* __\$arch_\$dir_\$file_H */
EOF

$/ = undef;

while(defined($_ = shift)) {
	my ($base, $arch, $dir, $file, $f, $str) = 
		m!^	(?:(.*)/)?			# Beginning path
			([^/]+)/			# (i386)
			([^/]+)/			# (kern)
			(.+?)    (?:\.[cho])?$!ix;	# File w/o extension
	$base ||= '.';

	unless(defined $dir and defined $arch and defined $file) {
		print STDERR "Wrong argument '$_'\n";
		next;
	}

	print "Please enter a short description for these files:\n";
	$/ = "\n";
	my $sdesc = <STDIN>;
	undef $/;
	@hash{qw/arch dir sdesc/} = ($arch, $dir, $sdesc);

	open C, "$base/$inc" or do {
				print STDERR "Can't open $base/$inc: $!\n";
				next
			};
	$f = <C>;
	close C;

	my $cf = "$base/$arch/$dir/$file.c";
	my $hf = $base."/include/$arch/$dir/$file.h";

	&do_file($cf, $f, 'c', $file, $endc);
	&do_file($hf, $f, 'h', $file, $endh);
}

sub trans {
	my ($t, $f, $v) = @_;
	$t =~ s/\$file(\S)/$f$1/ig;
	$t =~ s/\$file/$f.$v/ig;
	$t =~ s/\$([0-9a-z]+)/$hash{$1}/ig;
	return $t;
}

# Horrid code - but it works.. Mark
sub do_file {
	my ($f, $text, $ty, $file, $end) = @_;
	if(-e $f) {
		print STDERR "$f exists. skipping\n";
	} else {
		open D, ">$f" or do {print STDERR "Can't open $f: $!\n";next};
		my @str;
		for(split /\n\n/, $text) {
			my @a = split /\\n\n/;
			for my $t (@a) {
				push @str, wrap((' * ') x 2, trans($t,$file,$ty));
			}
		}
		my $str = join "\n", @str;
		$str =~ s/^$/ * /gms;
		print D "/*\n", $str, trans($end,$file,$ty), "\n";
		close D;
	}
}
