#!/usr/bin/perl -w
# Last update: 12/12/01
use strict;
use File::Find;
use IO::Handle;
require 'tools/Parse.pm';		# Parses mod_depend and config

# We parse these options from the user, These numbers corrospond to the bit to
# set. These must be the same as the numbers defined in
# include/generic/kern/debug.h. - Mark
my %compile_hash = (
		Y => [ 0 ],
		D => [ 0, 1 ],
		M => [ 0, 2 ],
		N => [],
	);

my $config = 'config';
my $mod_depend = 'mod_depend';
my $defaults = 'defaults';
my @conf_overrides = qw/GCC/;		# List of %defaults that can be
					# overrided from %config
my @opts;

my %user_override;			# The highest level of overrides
					# (specified on the command-line)
STDOUT->autoflush(1);			# A *MUST* for 'make' in vim to work ;)

while(defined($_ = shift @ARGV)) {
	/^-T(.*)=(.*)/ or do {
		unshift @opts, $_;
		next
	};
	$user_override{$1} = $2;
}

my %defaults = Parse::Config($defaults);
$defaults{$_} = $user_override{$_} for keys %user_override;
my %config = ();

if(!-f $config) {
	if(shift @opts eq 'clean') {
		require $defaults{ARCH}.'/tools/mkboot.pl';
		&clean;
		exit unless @ARGV
	} else {
		print STDERR "Please run 'make config' to generate your config file\n";
		exit 1
	}
}

%config = Parse::Config($config);
%config = (%defaults, %config);
exists $config{$_} and $defaults{$_} = $config{$_} for @conf_overrides;

$config{$_} = $user_override{$_} for keys %user_override;

#print "$_ => $config{$_}\n" for keys %config;

require $config{ARCH}.'/tools/mkboot.pl';

my %mod_depend = Parse::Mod_deps($config{ARCH},$mod_depend);
my %t = Parse::Mod_deps('generic',$mod_depend);
push @{$mod_depend{$_}}, @{$t{$_}} for keys %t;

my %modules;

unlink 'include/arch';
symlink $config{ARCH}, 'include/arch' or
		die "Can't symlink include/$config{ARCH} to include/arch";

my $gcc_ver;
chomp($gcc_ver = (`$defaults{GCC} -v 2>&1`)[1]);

open F, "> $defaults{AUTO_DEFINE}" or die "Can't open $defaults{AUTO_DEFINE} " .
							"for writing, $!";

print F <<EOF;
/* This file is automatically regenerated every time you make zealos. Please
 * make changes elsewhere (config, mod_depend, etc)
 */
#define COMPILE_USER "$ENV{USER}"
#define COMPILE_TIME "@{[scalar localtime]}"
#define COMPILE_HOST "$ENV{HOSTNAME}"
#define COMPILE_VER "$gcc_ver"

EOF

my $d = (exists $config{DEBUG} and lc $config{DEBUG} eq 'y')?1:0;
for(keys %config) {
	if($config{$_} =~ /^([mdyn]+)$/i) {
		my $num = 0;
		for my $f (split //, uc $config{$_}) {
			next unless exists $compile_hash{$f};
			$num |= 1 << $_ for @{$compile_hash{$f}}
		}

		print F "#define CONFIG_$_ $num\n" if $num;

		my $f;

		if(lc $config{$_} eq 'y' or lc $config{$_} eq 'd') {
			for $f (@{$mod_depend{$_}}) {
				$modules{loader}{$f} = $num
			}
		} elsif($config{$_} =~ /m/i) {
			for $f (@{$mod_depend{$_}}) {
				$modules{$_}{$f} = $num
			}
		}
	} else {
		s/-/_/g;
		my $t = $config{$_};
		$t =~ s/-/_/g;
		print F "#define __${t}__\n" if $_ eq 'ARCH';
		print F "#define $_ $t\n";
		print F "#define ${_}_STR \"$t\"\n";
	}
}
close F;

for my $arg (@opts) {
	if(lc $arg eq 'clean') {
		&clean;
	} elsif($arg eq 'modules') {
		makeModule($_) for grep {$_ ne 'loader'} keys %modules;
	} elsif(exists $modules{$arg}) {
		makeModule($arg);
	} elsif($arg eq 'install') {
		my $t = &inter($defaults{IMAGE});
		if(-e $t) {
			my $f = &inter($config{INSTALL});
			print "Copying zealos image to $f: ";
			if(system 'cp', $t, $f) {
				print "Failed\n";
				exit 1;
			}
			print "done\n";
		} else {
			print "$t doesn't exist try 'make'\n";
		}
	} else {
		print STDERR "'$arg' is not a recognised option\n";
	}
}
exit;

sub makeModule {
	my $arg = shift;

	my @files = grep /\.c$/, keys %{$modules{$arg}};

	for(@files) {
		my ($type) = m!([^/]+)/[^/]+$!;

		my @inc = ("arch/$type", "generic/$type", "arch/kern",
					"generic/kern", "arch", "generic","");
		my %seen;		# Remove dupes
		@inc = grep {!$seen{$_}++} @inc;

		my $gccstr = "$defaults{GCC} $defaults{GCC_FLAGS} " .
			' -D' . (($arg eq 'loader')?'__LOADER__':'__MODULE__') .
			" -DCURRENT=$modules{$arg}{$_} " .
			join(' ', map {$_="-Iinclude/$_"} @inc);
		inter(\$gccstr);
		make($_, $gccstr);
	}

	s/\.c$/.o/ for @files;

	my ($ldstr, $file);
	if($arg eq 'loader') {
		$file = "\$ARCH/boot/\$LOD_FILE";
		inter(\$file);
		$ldstr = "$defaults{LD} $defaults{LOADER_LD_FLAGS} -o $file";
	} else {
		mkdir 'modules', 0755 unless -d 'modules';
		$ldstr = "$defaults{LD} $defaults{MODULE_LD_FLAGS} -o " .
							"modules/$arg.o";
	}
	$ldstr .= ' ' . join ' ', @files;
	inter(\$ldstr);
	print "$ldstr\n";
	(system $ldstr) == 0 or die "link failed";
	if($arg eq 'loader') {
		chdir "$config{ARCH}/boot" or die "Can't chdir to " .
					"$config{ARCH}/boot, $!";
		no strict 'refs';
		&{$config{BOOT}}($config{LOD_FILE}, "../../$config{MAPFILE}",
				'../../' . &inter($defaults{IMAGE}), \%config);
		use strict 'refs';
	}
}

sub inter {
	my $t = shift;
	if(ref $t eq 'SCALAR') {
		$$t = &__inter($$t);
	} else {
		return &__inter($t);
	}
}

sub __inter {
	my $t = shift;
	$t =~ s!\$(\w+)!
		(exists $config{$1})?$config{$1}:
			((exists $defaults{$1})?$defaults{$1}:'')!exg;
	return $t
}

sub make {
	my ($file, $str) = @_;
	my $obj = $file;
	$obj =~ s/\.c$/.o/;
	$str .= " -o $obj $file";

	# Prevent needless compilation
	return if -e $obj and (stat $obj)[9] > (stat $file)[9];

	print "$str\n";		# print it...
	(system $str) == 0 or die "failed";		# do it!!
}

sub clean {
	find(sub{-f and /.o$/ and unlink}, './');
	find(sub {-f and unlink; -d and rmdir}, 'modules')
							if -d 'modules';
	rmdir 'modules';
	unlink 'include/arch';
	unlink $defaults{MAPFILE}, "$config{ARCH}/boot/$defaults{LOD_FILE}",
		$defaults{AUTO_DEFINE};
	unlink &inter($defaults{IMAGE});
	chdir "$config{ARCH}/boot" or die "Can't chdir to ".
				"$config{ARCH}/boot, $!";
	&bootClean;
}
