#!/usr/bin/perl
# mkdoc.pl, a program to make documentation for ZealOS (Copyright Mark Zealey,
# 2001 but GPL'd)
# Last update: 27/06/01

# This will look thru all the .c and .h files in the ZealOS tree and get the
# comments out of them. It will then look for certian keywords in those comments
# and parse them out to get function info. It will then dump all this info into
# one text file.

# This program is looking through for some comment like:
# /* Function: foo(int bar, void *fred, char **foo)
#  * arg0: bar is any old value
#  * arg2: foo is a pretty stupid thing
#  * type: local
#  * Description: This is my function foo, which if I were to put in the main
#  * kernel, I would get shot by Mark because it's stupid to name a global
#  *              function 'foo'...
#  */

# (you can use 'function', 'macro', 'struct', 'variable' or 'typedef' too)

# As you will see, it's very flexable, so just try it and see.. ;)

use strict;
use File::Find;
use Cwd;
use Text::Wrap qw/fill wrap $columns/;
require 'tools/Doc.pm';
require 'tools/Parse.pm';

$columns = 80;
my $indent = "    ";
my $func_indent = $indent;
my $arg_indent = $indent x 3;
my $para_indent = $indent x 2;

# Get module files
my %config = &Parse::Config;
my %mod_depend = &Parse::Mod_deps($config{ARCH},'mod_depend');
my %t = Parse::Mod_deps('generic','mod_depend');
push @{$mod_depend{$_}}, @{$t{$_}} for keys %t;

for my $module (sort keys %mod_depend) {
	my %file = Doc::get_docu($module);

	next unless keys %file;

	print '=' x $columns, "\n";
	print ' ' x (($columns - length $module) / 2), $module, "\n";
	print '=' x $columns, "\n";

	for my $file (keys %file) {
		next unless keys %{$file{$file}} > 1;
		print "\n$file:\n";
		for (sort {$file{$file}{$a}{type} cmp $file{$file}{$b}{type}}
							keys %{$file{$file}}) {
			next unless $_;
			&print_line($file{$file}{$_}, $_);
			print "\n\n";
		}
	}
}

sub print_line {
	my ($cur, $func) = @_;

	my @list = split /\n/, $func;
	
	if(@list > 1) {
		my $t;
		$t .= "$_ $cur->{calling_list}\n" for @list;
		print wrap($func_indent,$func_indent, $t);
		print wrap(($para_indent)x2, "These are", uc $cur->{type}.'s',
					"of type", uc $cur->{scope}), "\n\n";
	} else {
	# Print the function name and arguments
		print wrap($func_indent,$func_indent.($indent x 4),
					"$func $cur->{calling_list}"),"\n";
		print wrap(($para_indent)x2, "This is a", uc $cur->{type}
				, "of type", uc $cur->{scope}), "\n\n";
	}

	# Print a description of each argument
	print $para_indent,"Arguments:\n" if keys %{$cur->{args}};
	for(sort {$a cmp $b} keys %{$cur->{args}}) {
		my $s = "Argument@{[/to/ && 's']} $_: $cur->{args}{$_}";
		print wrap("$arg_indent* ","$arg_indent  ", $s),"\n";
	}
	print "\n" if keys %{$cur->{args}};

	# Print the description
	$cur->{description} =~ s/( {2,})/'<sp'.(length$1).'>'/egg;
	$cur->{description} =~ s/(\t+)/'<sp'.((length$1)*8).'>'/egg;
	my $p = fill($para_indent.$indent, $para_indent,
					split /\n/, $cur->{description});
	$p =~ s/<sp(\d+)>/' ' x $1/egg;
	print $p;
}
