; An include file for #define's for asm - Mark
; Last update: 09/07/01

; This loads, saves the lilo command line, and then copies itself to BOOT_MOVE.
; It then sets up the registers and the stack. A memory dump looks like this:
; 0x10000+		Where the loader is loaded :)
; Below this		The paging tables are set up just below the sec loader
; Just below 0x9F000	Where we load the secondary loader to (4 sects)
; 0x9F000 - 0x9FC00	Reserved for options to the kernel :)
; 0x9FC00 - 0x9FCFF	Where the lilo command line is copied to
; 0x9FD00 - 0x9FEFF	Copied bootloader (We can overwrite all this)
; 0x9FF00 - 0x9FFFE	Stack area
; 0xA0000 - 0xFFFFF	Memory mapped IO area. We should leave this alone.

%define SEC_SIZE 4		; SIZE of the secondary loader (sectors)

%define BASE_SEG 0x9000
%define MEM_LOC_MAX_SZE 0x200	; Max size of the data struct
; Where we pass the data to the loader in (MUST BE THE SAME AS IN INIT.H!!!!)
%define MEM_LOC_SEG 0x9F00
%define DATA_AREA 0xFC00	; Where the lilo string is

; Where the secondary loader is loaded to (seg)
%define SEC_LOAD (MEM_LOC_SEG - (512 * SEC_SIZE) / 16)
; 1024 entrys in the table (4 bytes each)
%define PG_TBL_ENT 1024
%define PG_TBL ((((SEC_LOAD * 16) / 4096) - 1) * 4 * PG_TBL_ENT)
; 1024 entrys in the directory (4 bytes each)
%define PG_DIR (PG_TBL - 4096)

%define BOOT_MOVE 0xFD00	; Where the bootloader moves to (in bytes)
%define STACK_START 0xFFFE	; FAKE: we also put the floppy patch onto stack
%define STACK_P (STACK_START - FLOPPY_P_SZE)	; Where the stack is (ended)
%define FLOPPY_P_OFF (STACK_P+1); Where to load the new floppy info (top of
				;	stack)
%define FLOPPY_P_SZE 12		; Size to load (bytes)

; OK, the data is in 0x9F000, here are the offsets (MUST BE THE SAME AS IN
; INIT.H!!!!!!!) (All values are ulongs unless otherwise stated)
%define O_KERN_SIZE	0x0		; Kernel size
%define O_MEM_SIZE	0x4		; Size of the whole of memory
%define O_LILO		0x8		; Pointer to the lilo string
%define O_MAP_ADDR	0xC		; Pointer to the mmap string
%define O_MAP_LEN	0x10		; Length of the string
%define O_KERN_START	0x14		; Where the kernel starts

%define O_SYMS_START	0x18		; Where the symbols start
%define O_SYMS_LEN	0x1C		; Length of them
%define O_DELABLE_START	0x20		; Where the initalization temp stuff
					; starts
%define O_DELABLE_LEN	0x24		; Length of it
%define O_FS_START	0x28		; Where the FS Structs start
%define O_FS_LEN	0x2C		; Length of the section
%define O_SYSCALL_START	0x30		; Where the syscall Structs start
%define O_SYSCALL_LEN	0x34		; Length of the section
%define O_STACK_START	0x38		; Start address of the stack

%define O_MAP_OFF	0x100		; Offset of memory map
%define MAP_OFF_LEN	0x100		; Max length

; Detecting of parameters doesnt work on some floppy disks/BIOSes. Define this
; to autodetect
;%define USE_BIOS_AUTO
