/*
 * 8259.c - A generic 8259 Programmable IRQ controller
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 29/06/01
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 * 06/07/00 Mark: Many thanks to 'believer' who made this work ;-)
 * 27/06/01 Mark: Moved most of the stuff over to generic/
 */

#include <common.h>
#include <irq.h>
#include <io.h>
#include <stdio.h>

#ifdef DEBUG_IRQ
#include <printk.h>
#endif


// Number of IRQ's we have
#define IRQ_NUM 16

// Start values with the controller line to slave enabled
static uchar masks[IRQ_NUM/8] = { ~(1 << 2), 0xFF };

ulong __preInitFunc IRQInitArch()
{
	outb_p(0x11, 0x20);			// init
	outb_p(0x11, 0xA0);			// init
	outb_p(IRQ_OFFSET, 0x21);		// offsets
	outb_p(IRQ_OFFSET + 8, 0xA1);		// offsets
	outb_p(4, 0x21);			// cascading
	outb_p(2, 0xA1);			// cascading
	outb_p(1, 0x21);			// 80x86 mode
	outb_p(1, 0xA1);			// 80x86 mode

	outb_p(masks[0], 0x21);
	outb_p(masks[1], 0xA1);

	return IRQ_NUM;		// Number of IRQ's we've got
}

void EOIIntr(ulong IRQ)
{
	/* We used to have it so it just EOI'd all, but that would mess w/ the
	 * queuing, so now we mask a specific IRQ */
	if(IRQ > 7) {			// Second PIC
		outb(0x60 + (IRQ & 7), 0xA0);	// IRQ number
		EOIIntr(2);			// EOI the first PIC
	} else
		outb(0x60 + IRQ, 0x20);
}

errRet IRQMask(ulong IRQ, bool enable)	// 1 to enable, 0 to disable
{
	uchar chip = 0;

	if(IRQ >= IRQ_NUM)		// Out of range
		return ETOOBIG;

	EOIIntr(IRQ);			// Clear any pending (XXX: This needed?)

	if(IRQ > 7) {
		IRQ -= 8;
		chip++;
	}

	if(enable == ENABLE)		// Enable
		btc(&masks[chip], IRQ);
	else				// Disable
		bts(&masks[chip], IRQ);

	outb(masks[chip], (chip == 1)?0xA1:0x21);
#ifdef DEBUG_IRQ
	DPRINT("IRQ: %d, chip: %d, mask: 0x%X\n", IRQ, chip, masks[chip]);
#endif

	return SUCCESS;
}

EXPORT_SYMBOL(IRQMask);
EXPORT_SYMBOL(EOIIntr);
