/*
 * delay.c - Delay a set period of time
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 11/07/01
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 */

#include <common.h>
#include <delay.h>
#include <printk.h>
#include <cpu.h>
#include <math64.h>


/* function: void __delay(ulong loops)
 * type: Global
 * description: Delays for 'loops' loops. You probably want 'delay' instead.
 */
void __delay(ulong loops)
{
#if PROCESSOR >= 5
	ulong b, c;
#endif
	if(!loops)
		return;		// Prevent infinate loops.
#if PROCESSOR >= 5
	rdtscl(b);
	do rdtscl(c) while(c - b < loops);
#else
	// Varies wildly depending on alignment, force 4-byte alignment so it
	// doesn't.
	asm volatile (".align 16; 1: decl %0; jnz 1b" : : "r" (loops));
#endif
}

void __udelay(ulong microsec)
{
	__uint32 upper, lower;

	if(microsec > 999) {		// Prevent overflows
		__uint32 milisec = microsec / 1000;
		delay(milisec);
		microsec -= milisec * 1000;	// Do the rest (XXX: should we?)
	}

	/* OK, this code is clever (It took me about a day to understand from
	 * linux's, and then I changed it a bit). First, we multiply microsec by
	 * 2 << 32, because when we do the Mul64, we loose the end-long, which
	 * is the equivelent of dividing by 2 << 32. Then, we divide that number
	 * by 1000, because there's 1000 microseconds in a milisecond. After
	 * that, we do the multiply against the number of loops in a milisecond,
	 * and then we have our delay. (Hmm, just noticed how this comment is
	 * longer than the code itself...)
	 */
	from64(mathMul64(
				microsec * 4294967 /* ((2 << 32) / 1000) */,
				curCPU->loopsPerMs),
			upper,
			lower);
	__delay(upper);
}

EXPORT_SYMBOL(__delay);
EXPORT_SYMBOL(__udelay);
