/*
 * init.c - Do x86-64 specific kernel initalizations (with things taken from the
 *	bootloader)
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 29/06/01
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 */


#include <common.h>
#include <cpu.h>
#include <init.h>
#include <stdio.h>
#include <printk.h>

struct basicValues initVals;
struct memMap mminfo[MMAP_ENTRIES];
struct bloaderVals *bootstrapVals = (struct bloaderVals *)ARG_AREA;

/* This retreves and parses the command line parameters passed by lilo at boot
 * time.
 */
initFunc __preInitFunc parseLilo()
{
	__uint8 k = MAX_COMMAND_LEN-1;
	/* A buffer of the options passed by lilo */
	char i[MAX_COMMAND_LEN], *j, *sp=i, *end=i;

	if(!initVals.commandLine[0]) {		// Lilo loaded us?
		printk("Looks like we wern't loaded by lilo\n");
		// Not having lilo isnt an error... we just cant be bothered
		// to process nothing ;)
		return SUCCESS;
	} else
		printk("Looks like lilo loaded us\n");
	
	kmemcopy(i, initVals.commandLine, MAX_COMMAND_LEN);

	while(*end && k--)
		end++;

	while(sp < end) {
		/* Move along to the next space */
		j = sp;
		/* Move sp along to the next space */
		while((*sp != ' ') && *sp) sp++;

		/* We now change the space to a null so the following work OK */
		*(sp++) = 0;
		if(kbeginswith("print=", j))
			printk("%s", j+6);
		if(kbeginswith("mem=", j))
			thisBox.memCnt = atoi(j+4, 10) * 1024;
	}
	return SUCCESS;
}

// Inits some values, copys the data we got passed from the loader
initFunc __preInitFunc initInit()
{
	struct memMap *t;
	__uint64 bytes = 0;		// Memory size in bytes

	initVals.kernelLength = bootstrapVals->kernSize;
	initVals.kernelStart = bootstrapVals->kernStart;
	initVals.symsStart = bootstrapVals->symsStart;
	initVals.symsLen = bootstrapVals->symsLen;
	initVals.delableStart = bootstrapVals->delableStart;
	initVals.delableLen = bootstrapVals->delableLen;
	initVals.fsStart = bootstrapVals->fsStart;
	initVals.fsLen = bootstrapVals->fsLen;
	initVals.syscallStart = bootstrapVals->syscallStart;
	initVals.syscallLen = bootstrapVals->syscallLen;
	initVals.stackStart = bootstrapVals->stackStart;

	kmemset((char *)mminfo, 0, MMAP_LEN);

	if(bootstrapVals->mmapLen) {	// Woppie!! We've got a memory map!! >:)
		// Copy it (we'll process it later :)
		kmemcopy((char *)mminfo, (char *)bootstrapVals->mmapStr,
				bootstrapVals->mmapLen);
	} else {		// We disallow the range 0xA0000..0xFFFFF
		mminfo[0].base = 0x0;		// Frst 640Kb
		mminfo[0].len = 0xA0000;
		mminfo[0].type = 1;
		mminfo[1].base = 0xA0000;	// MMIO area
		mminfo[1].len = 0x5FFFF;
		mminfo[1].type = 2;
		mminfo[2].base = 0x100000;	// The rest of memory
		mminfo[2].len = bootstrapVals->memSize * 1024;
		mminfo[2].type = 1;
	}

	// Cycle thru the list now, looking at the sizes and adding them
	// togeather so we get a memory size
	for(t=mminfo; t->len; t++)
		bytes += t->len;

	// Memory starts from 0, so adjust by 1 for the purpose of.. well..
	// self-satisfaction ;)
	//
	// This used to be /1024, but gcc 2.7 versions couldn't optomize it out
	// properly, so I changed it to >> 10.
	thisBox.memCnt = ++bytes >> 10;		// memory size in Kb

	// Copy the command line now
	kmemcopy(initVals.commandLine, bootstrapVals->liloStr,MAX_COMMAND_LEN);
	return SUCCESS;
}
