/*
 * pit.c - Programmable Interrupt Timer for x86-64
 *
 * Added to the Zealos project and (C) by Mark Zealey
 *
 * Current maintainer: Mark Zealey <mark@itsolve.co.uk>
 * Status: Maintained
 * Last update: 29/06/01
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Description:
 *
 * Notes/Fixme's:
 *
 * Changelog:
 */

#include <common.h>
#include <io.h>

#ifdef DEBUG_PIT
#include <printk.h>
#endif

/* This changes the PIT to trigger an interrupt once every <delay> microseconds.
 * note the miliseconds. it should be enough, but out PIT is capable of 1193180
 * Hz precisly. - Mark
 * NB: each pic (Yes, there are 2) has 3 channels, starting from 0.
 */
void pitSet(__uint16 delay, __uint8 channel)
{
	__uint32 t = 1193180/delay;
	__uint8 port = 0x40;		// Default port

	if(channel > 2) {		// Change if it's for the second PIT
		port += 8;
		channel -= 3;
	}

	
#ifdef DEBUG_PIT
	printk(KINFO "Set PIT delay to %lu on port 0x%lx\n", (ulong)t,
			(ulong)(port+channel));
#endif

	outb((channel << 6) + 0x36, port + 3);    // Load control word into PIT
	outb((__uint8)(t & 0xFF), port + channel); // load low count value
	outb((__uint8)(t >> 8), port + channel);  // load high count value
}

EXPORT_SYMBOL(pitSet);
